;   ***************************************************************
;   * Copyright (c) 2002, Embed Inc (http://www.embedinc.com)     *
;   *                                                             *
;   * Permission to copy this file is granted as long as this     *
;   * copyright notice is included in its entirety at the         *
;   * beginning of the file, whether the file is copied in whole  *
;   * or in part and regardless of whether other information is   *
;   * added to the copy.                                          *
;   *                                                             *
;   * The contents of this file may be used in any way,           *
;   * commercial or otherwise.  This file is provided "as is",    *
;   * and Embed Inc makes no claims of suitability for a          *
;   * particular purpose nor assumes any liability resulting from *
;   * its use.                                                    *
;   ***************************************************************
;
;   Include file for the PRG PIC application.
;
;   This application controls the PICster low cost PIC programmer.
;   It runs on a PIC 16F628, which communicates with a host via
;   RS-232 and controls all the local programmer electronics.
;   See the FW_PRG documentation file for more details.
;
         include "prglib.inc"
;
;   General firmware information reported by VERS command.
;
info_org equ     2           ;1-254 organization ID, 2 = Philip Pemberton
info_cvlo equ    2           ;lowest spec version backward compatible with
info_cvhi equ    8           ;highest spec version compatible with
info_vers equ    8           ;1-254 private version number within organization
info_info equ    h'57693438' ;private 32 bit information about this firmware
                                ; 0x57693438 = ASCII "Wi48" = Wisp628 with
								;   PIC16F648 and Easyprog firmware kit
info_fwi2 equ    h'00'       ;fwinfo2 byte
;
;   Application configuration parameters
;
t2pms    equ     5           ;timer 2 ticks per millisecond, verified in PRG_INTR
to_hostms equ    100         ;10-2500 mS indicate host communication after receive
to_tronms equ    200         ;10-2500 mS indicate target on since last on
to_writms equ    250         ;10-2500 mS indicate writing since last write
to_houtms equ    5000        ;100-25000 mS host inactivity reset, 0 = disable

gbank    equ     0           ;direct register bank for global variables
;
;   Derived constants.
;
to_host  equ     (to_hostms + 5) / 10 ;host comm indicator time, 10mS units
to_tron  equ     (to_tronms + 5) / 10 ;target on indicator time, 10mS units
to_writ  equ     (to_writms + 5) / 10 ;write indicator time, 10mS units
to_hout  equ     (to_houtms + 50) / 100 ;host receive reset timeout, 100mS units

gbankadr equ     bankadr(gbank) ;address within reg bank for global variables
;
;   Response byte values.  Responses are sent to the host over the serial
;   line.  Note that some of these are sent asynchronously, and not really
;   in response to anything sent by the host.
;
;   Multi-byte numeric parameters following response codes are always sent in
;   least to most significant byte order.
;
rsp_ack  equ     1           ;last command acknowledged, clear to send next
;
;**********
;
;   Global flag bits.  As many GFL0 thru GFLn variables as needed are
;   automatically created by the /FLAG preprocessor directive.  After all
;   flags are defined, NFLAGB will be left indicating the number of GFLx
;   variables created.  For each flag, the string substitution macro
;   flag_<name> is defined to be "gfl<n>,<bit>".  The flag_<name> symbol
;   can be used directly with bit instructions.
;

; /flag    sin                 ;a serial line input byte is available
nflagb set 1
flag_sin_regn equ 0
#define flag_sin_reg gfl0
flag_sin_bit equ 0
#define flag_sin gfl0,0

; /flag    sout                ;serial output can accept another byte
flag_sout_regn equ 0
#define flag_sout_reg gfl0
flag_sout_bit equ 1
#define flag_sout gfl0,1

; /flag    ledph1              ;in LED phase 1, not 2
flag_ledph1_regn equ 0
#define flag_ledph1_reg gfl0
flag_ledph1_bit equ 2
#define flag_ledph1 gfl0,2

; /flag    done                ;wait interval has elapsed
flag_done_regn equ 0
#define flag_done_reg gfl0
flag_done_bit equ 3
#define flag_done gfl0,3


; /flag    tdrive              ;target is driving the PGD (serial data) line
flag_tdrive_regn equ 0
#define flag_tdrive_reg gfl0
flag_tdrive_bit equ 4
#define flag_tdrive gfl0,4

; /flag    waitcmd             ;suspend command processing thread until wait over
flag_waitcmd_regn equ 0
#define flag_waitcmd_reg gfl0
flag_waitcmd_bit equ 5
#define flag_waitcmd gfl0,5

; /flag    host                ;currently communicating with the host
flag_host_regn equ 0
#define flag_host_reg gfl0
flag_host_bit equ 6
#define flag_host gfl0,6


; /flag    vppon               ;target Vpp programming voltage is on
flag_vppon_regn equ 0
#define flag_vppon_reg gfl0
flag_vppon_bit equ 7
#define flag_vppon gfl0,7

; /flag    trgon               ;indicate target chip is "on", don't remove it
nflagb set 2
flag_trgon_regn equ 1
#define flag_trgon_reg gfl1
flag_trgon_bit equ 0
#define flag_trgon gfl1,0

; /flag    write               ;currently writing to the target chip
flag_write_regn equ 1
#define flag_write_reg gfl1
flag_write_bit equ 1
#define flag_write gfl1,1

; /flag    htimeout            ;host communication timed out, abort and reset
flag_htimeout_regn equ 1
#define flag_htimeout_reg gfl1
flag_htimeout_bit equ 2
#define flag_htimeout gfl1,2


; /flag    spdata              ;read/write data memory, not program memory space
flag_spdata_regn equ 1
#define flag_spdata_reg gfl1
flag_spdata_bit equ 3
#define flag_spdata gfl1,3

; /flag    tadrkn              ;target address is known and stored in TADRI
flag_tadrkn_regn equ 1
#define flag_tadrkn_reg gfl1
flag_tadrkn_bit equ 4
#define flag_tadrkn gfl1,4

; /flag    op1                 ;private flag used during table-selected operation
flag_op1_regn equ 1
#define flag_op1_reg gfl1
flag_op1_bit equ 5
#define flag_op1 gfl1,5

; /flag    adrinc              ;target address has been incremented since reset
flag_adrinc_regn equ 1
#define flag_adrinc_reg gfl1
flag_adrinc_bit equ 6
#define flag_adrinc gfl1,6


; /flag    pumpstate           ; Charge pump state
flag_pumpstate_regn equ 1
#define flag_pumpstate_reg gfl1
flag_pumpstate_bit equ 7
#define flag_pumpstate gfl1,7

; /flag    htimeoutdis         ; Host timeout disabled if =1
nflagb set 3
flag_htimeoutdis_regn equ 2
#define flag_htimeoutdis_reg gfl2
flag_htimeoutdis_bit equ 0
#define flag_htimeoutdis gfl2,0

; /flag    iosenab             ; I/Os in Hi-Z if =1
flag_iosenab_regn equ 2
#define flag_iosenab_reg gfl2
flag_iosenab_bit equ 1
#define flag_iosenab gfl2,1
;
;**********
;
;   I/O pin declaration.  Three constants are declared for each I/O pin.
;   xxx_REG is the register containing the bit for the pin, xxx_TRIS is the
;   corresponding tri-state enable register, and xxx_BIT is the number of
;   the bit within the registers.  All undelcared I/O bits will be configured
;   as outputs and driven low, to the extent allowed by the particular
;   processor.
;
portbpullups equ true        ;the port B weak pullups will be enabled


; /outbit  pgc     porta 0 0   ;serial programming clock line
pgc_reg equ porta
  ifdef trisa
pgc_tris equ trisa
    endif
pgc_bit equ 0
val_trisa set val_trisa & ~(1 << 0)
val_porta set val_porta & ~(1 << 0)
#define pgc_pin porta,0
  ifdef lata
pgc_lat equ lata
#define pgc_pinlat lata,0
    endif

; /inbit   pgd     portb 5     ;serial prog data line comparator input
pgd_reg equ portb
  ifdef trisb
pgd_tris equ trisb
    endif
pgd_bit equ 5
val_trisb set val_trisb | (1 << 5)
val_pullupb set val_pullupb  & ~(1 << 5)
#define pgd_pin portb,5
  ifdef latb
pgd_lat equ latb
#define pgd_pinlat latb,5
    endif

; /outbit  lvpen   portb 7 0   ; LVP enable (output)
lvpen_reg equ portb
  ifdef trisb
lvpen_tris equ trisb
    endif
lvpen_bit equ 7
val_trisb set val_trisb & ~(1 << 7)
val_portb set val_portb & ~(1 << 7)
#define lvpen_pin portb,7
  ifdef latb
lvpen_lat equ latb
#define lvpen_pinlat latb,7
    endif


; /outbit  pump1   portb 3 0   ; Charge pump pin 1
pump1_reg equ portb
  ifdef trisb
pump1_tris equ trisb
    endif
pump1_bit equ 3
val_trisb set val_trisb & ~(1 << 3)
val_portb set val_portb & ~(1 << 3)
#define pump1_pin portb,3
  ifdef latb
pump1_lat equ latb
#define pump1_pinlat latb,3
    endif

; /outbit  pump2   portb 0 0   ; Charge pump pin 2
pump2_reg equ portb
  ifdef trisb
pump2_tris equ trisb
    endif
pump2_bit equ 0
val_trisb set val_trisb & ~(1 << 0)
val_portb set val_portb & ~(1 << 0)
#define pump2_pin portb,0
  ifdef latb
pump2_lat equ latb
#define pump2_pinlat latb,0
    endif

; /outbit  rset    porta 1 0   ; VPP=0 pulldown (1=vpp grounded)
rset_reg equ porta
  ifdef trisa
rset_tris equ trisa
    endif
rset_bit equ 1
val_trisa set val_trisa & ~(1 << 1)
val_porta set val_porta & ~(1 << 1)
#define rset_pin porta,1
  ifdef lata
rset_lat equ lata
#define rset_pinlat lata,1
    endif


; /inbit   rx      portb 1     ;UART input
rx_reg equ portb
  ifdef trisb
rx_tris equ trisb
    endif
rx_bit equ 1
val_trisb set val_trisb | (1 << 1)
val_pullupb set val_pullupb  & ~(1 << 1)
#define rx_pin portb,1
  ifdef latb
rx_lat equ latb
#define rx_pinlat latb,1
    endif

; /inbit   tx      portb 2     ;UART output, must be declared as in input
tx_reg equ portb
  ifdef trisb
tx_tris equ trisb
    endif
tx_bit equ 2
val_trisb set val_trisb | (1 << 2)
val_pullupb set val_pullupb  & ~(1 << 2)
#define tx_pin portb,2
  ifdef latb
tx_lat equ latb
#define tx_pinlat latb,2
    endif


; /inbit   serin   portb 4     ; Serial Passthrough input
serin_reg equ portb
  ifdef trisb
serin_tris equ trisb
    endif
serin_bit equ 4
val_trisb set val_trisb | (1 << 4)
val_pullupb set val_pullupb  & ~(1 << 4)
#define serin_pin portb,4
  ifdef latb
serin_lat equ latb
#define serin_pinlat latb,4
    endif

; /outbit  serout  portb 6 0   ; Serial Passthrough output
serout_reg equ portb
  ifdef trisb
serout_tris equ trisb
    endif
serout_bit equ 6
val_trisb set val_trisb & ~(1 << 6)
val_portb set val_portb & ~(1 << 6)
#define serout_pin portb,6
  ifdef latb
serout_lat equ latb
#define serout_pinlat latb,6
    endif


; /inbit   debug   porta 5     ;the debug jumper is installed
debug_reg equ porta
  ifdef trisa
debug_tris equ trisa
    endif
debug_bit equ 5
val_trisa set val_trisa | (1 << 5)
val_pullupa set val_pullupa  & ~(1 << 5)
#define debug_pin porta,5
  ifdef lata
debug_lat equ lata
#define debug_pinlat lata,5
    endif

; /inbit   pgdpd   porta 3     ; pgd pull-down, not used  **REMOVEME
pgdpd_reg equ porta
  ifdef trisa
pgdpd_tris equ trisa
    endif
pgdpd_bit equ 3
val_trisa set val_trisa | (1 << 3)
val_pullupa set val_pullupa  & ~(1 << 3)
#define pgdpd_pin porta,3
  ifdef lata
pgdpd_lat equ lata
#define pgdpd_pinlat lata,3
    endif

;
;***********************************************************************
;
;   Macro MAKE_VDD_PWM <millivolts>
;
;   Set the assembler variable VAL_VDD_PWM to the closest PWM duty
;   cycle value to obtain the indicated target chip Vdd value in millivolts.
;   The available range is 0 to 6V, so the argument must be in the
;   range of 0 to 6000.
;
;   WARNING:  This macro relies on the PWM period being 250 instructions
;     This is set independently in the INTR module with the constants
;     T2PRE and T2PER.
;
make_vdd_pwm macro mv
val_vdd_pwm set  ((mv) * 251) / 6000
  if val_vdd_pwm < 0
val_vdd_pwm set  0
    endif
  if val_vdd_pwm > 250
val_vdd_pwm set  250
    endif
         endm
;
;***********************************************************************
;
;   Macro SETWAITMS ms
;
;   Start a wait interval that will be MS milliseconds long.  FLAG_DONE will
;   be initialized to cleared, and will automatically be set when the
;   wait interval elapses.  This macro only sets up the wait interval
;   and finishes "quickly".  The actual wait time is rounded up within the
;   available resolution to guarantee the wait time is never less than
;   MS milliseconds.  The special value of 0 for MS causes the shortest
;   possible wait of at least the time between two clock ticks (200uS).
;
;   Processing of host commands that require operations on the target chip
;   will be suspended until the wait interval elapses.
;
setwaitms macro  ms
         local   ii

ii       set     (ms)        ;get number of milliseconds to wait
  if ii <= 0
ii       set     0           ;clip at 0 to avoid negative wait time
    endif
ii       set     (ii * t2pms) + 1 ;make number of timer 2 ticks to wait
  if ii == 1                 ;0 wait time specified ?
ii       set     2           ;minimum CNTWAIT to guarantee some waiting
    endif

         dbankif gbankadr
         intr_off            ;temp disable interrupts
         loadk16 cntwait, ii ;set the timer 2 ticks down counter timer
         bcf     flag_done   ;init to wait interval not elapsed yet
         intr_on             ;re-enable interrupts
         endm
;
;***********************************************************************
;
;   Macro WAIT
;
;   Wait for the current wait interval to elapse.  This marco will
;   complete immediately if no wait interval is in progress, meaning
;   the previous wait has already elapsed.
;
wait     macro
         dbankif gbankadr
         btfss   flag_done   ;wait interval has elapsed ?
         goto    $-1         ;no, go back and check again
         endm
;
;***********************************************************************
;
;   Macro WAITMS ms
;
;   Wait the indicated number of milliseconds.  The wait time will be
;   rounded to the available resolution so that the actual wait is never
;   less than the amount specified by the argument.
;
;   This macro is just a convenience wrapper around the SETWAITMS and WAIT
;   macros.
;
waitms   macro   ms
         setwaitms ms        ;set up the start of a new wait interval
         wait                ;wait for it to elapse
         endm
;
;***********************************************************************
;
;   Macro WRITING
;
;   Indicate that a write operation is in progress.  This resets the
;   write timeout counter to its maximum time, which causes FLAG_WRITE
;   to be set, which in turn causes the status LED to indicate the
;   target is being written to.
;
writing  macro
         extern  cn_writ     ;write timer down counter, 10mS units

         dbankif gbankadr
         movlw   to_writ     ;get the write timer reset value
         movwf   cn_writ     ;reset the timer
         endm
