;   ***************************************************************
;   * Copyright (c) 2002, Embed Inc (http://www.embedinc.com)     *
;   *                                                             *
;   * Permission to copy this file is granted as long as this     *
;   * copyright notice is included in its entirety at the         *
;   * beginning of the file, whether the file is copied in whole  *
;   * or in part and regardless of whether other information is   *
;   * added to the copy.                                          *
;   *                                                             *
;   * The contents of this file may be used in any way,           *
;   * commercial or otherwise.  This file is provided "as is",    *
;   * and Embed Inc makes no claims of suitability for a          *
;   * particular purpose nor assumes any liability resulting from *
;   * its use.                                                    *
;   ***************************************************************
;
;   This module contains routines specific to handling 18F target
;   devices.  Unless otherwise noted, these routines assume the
;   target is in programming mode and ready to execute programming
;   commands.
;
         include "prg.inc"

         extern  sert_send   ;send REG8 low bits of REGA to the target
         extern  sert_recv   ;get REG8 bits from target into low bits of REGA
         extern  op_inc_tadr ;increment desired adr of next operation by 1

         extern  done_read   ;jump here when specific read routine done
         extern  done_write  ;jump here when specific write routine done

         extern  tadr        ;requested target chip address of operation
         extern  tadri       ;actual internal target chip address
         extern  cntwait     ;wait interval down counter
         extern  tprog       ;wait time after write in timer 2 ticks
         extern_flags        ;declare global flag bits EXTERN
;
;***********************************************************************
;
;   Configuration constants.
;
lbank    equ     0           ;register bank for the local state of this module
;
;   Derived constants.
;
lbankadr equ     bankadr(lbank) ;address within local state register bank
;
;***********************************************************************
;
;   Global state.  All this state is assumed to be in the GBANK register
;   bank by other modules.
;
.bank#v(gbank) udata

;
;***********************************************************************
;
;   Local state.
;
  if lbank != gbank
.bank#v(lbank) udata
    endif

.18f    code
;
;***********************************************************************
;
;   Local subroutine SEND20
;
;   Send the low 20 bits of REGA to the target.  The low 4 bits are
;   therefore the command, and the remaining 16 bits the data for
;   that command.
;
         locsub  send20, regf8

         movlw   20          ;pass number of bits to send
         movwf   reg8
         gcall   sert_send

         leaverest
;
;***********************************************************************
;
;   Macro CORE_INSTR instr
;
;   Cause the instruction core to execute the indicated 16 bit instruction.
;
;   REG0, REG1, and REG2 are trashed.
;
core_instr macro instr
         loadk24 reg0, ((instr) << 4) ;pass opcode with instruction as data
         mcall   send20      ;send it
         endm
;
;***********************************************************************
;
;   Local subroutine SETFSR
;
;   Set the target special function register at the address in REG3 to the
;   value in REG2.
;
         locsub  setfsr, regf0 | regf1 | regf8
;
;   Send a command to execute core instruction to load the data value into W.
;
         clrf    reg0        ;pass CORE INSTRUCTION command
         movlw   4           ;pass number of bits to send
         movwf   reg8
         gcall   sert_send   ;send the command

         movlw   h'0E'       ;high byte of MOVLW instruction
         movwf   reg1
         movf    reg2, w     ;low byte of MOVLW instruction is data value
         movwf   reg0
         movlw   16          ;number of bits to send
         movwf   reg8
         gcall   sert_send   ;send the data value for this command
;
;   Send a command to write the W register to the selected SFR.
;
         clrf    reg0        ;pass CORE INSTRUCTION command
         movlw   4           ;pass number of bits to send
         movwf   reg8
         gcall   sert_send   ;send the command

         movlw   h'6E'       ;high byte of MOVWF instruction
         movwf   reg1
         movf    reg3, w     ;low byte of MOVWF instruction is target address
         movwf   reg0
         movlw   16          ;number of bits to send
         movwf   reg8
         gcall   sert_send   ;send the data value for this command

         leaverest
;
;***********************************************************************
;
;   Local subroutine ADR18F
;
;   Set the internal target chip address to match the desired address.
;   The desired address is in TADR and the current address internal
;   to the target chip is TADRI.  Nothing is done if the internal
;   address already matches the desired address.
;
         locsub  adr18f, regf0 | regf2 | regf3

         dbankif gbankadr
         btfss   flag_tadrkn ;the target address is known ?
         goto    adr_upd     ;target address unknown, go set it
;
;   Check the target address against the desired address.
;
         dbankif gbankadr
         movf    tadr+0, w   ;compare byte 0
         xorwf   tadri+0, w
         movwf   reg0

         movf    tadr+1, w   ;compare byte 1
         xorwf   tadri+1, w
         iorwf   reg0, w
         movwf   reg0

         movf    tadr+2, w   ;compare byte 2
         xorwf   tadri+2, w
         iorwf   reg0, w

         skip_nz             ;target address does not match desired address ?
         goto    adr18f_leave ;address matches, nothing to do
;
;   Set the target TBLPTR register to the desired address and update
;   the local copy of the current target address to match.
;
adr_upd  unbank
         dbankif gbankadr
         bsf     flag_tadrkn ;indicate the target address will be known

         dbankif gbankadr
         movf    tadr+0, w   ;get the address byte 0
         movwf   tadri+0     ;update local copy of target address
         movwf   reg2        ;pass data value
         movlw   h'F6'       ;pass target register address to write to
         movwf   reg3
         mcall   setfsr      ;set value of the special function register

         dbankif gbankadr
         movf    tadr+1, w   ;get the address byte 1
         movwf   tadri+1     ;update local copy of target address
         movwf   reg2        ;pass data value
         movlw   h'F7'       ;pass target register address to write to
         movwf   reg3
         mcall   setfsr      ;set value of the special function register

         dbankif gbankadr
         movf    tadr+2, w   ;get the address byte 2
         movwf   tadri+2     ;update local copy of target address
         movwf   reg2        ;pass data value
         movlw   h'F8'       ;pass target register address to write to
         movwf   reg3
         mcall   setfsr      ;set value of the special function register

adr18f_leave unbank          ;common exit point
         leaverest
;
;***********************************************************************
;
;   Routine F18_READ
;
;   Read the target memory location addressed by TADR and return the
;   data in REG1,REG0.  Since a data word on this target is only 8
;   bits wide, REG1 will always be returned 0.
;
;   REG2, REG3 may be trashed.
;
         glbent  f18_read
savedregs set    regf8
         pushregs savedregs

         mcall   adr18f      ;make sure target address matches TADR

         loadk16 reg0, b'1001' ;pass opcode and dummy 8 data bits
         movlw   12          ;pass total number of bits to send
         movwf   reg8
         gcall   sert_send   ;send READ MEMORY AND POSTINCREMENT opcode + 8 clocks
         movlw   8           ;pass number of bits to receive
         movwf   reg8
         gcall   sert_recv   ;read the data bits into REGA

         dbankif gbankadr
         incf    tadri+0     ;increment internal address low byte
         skip_nz             ;no carry to middle byte ?
         incf    tadri+1     ;increment internal address middle byte
         skip_nz             ;no carry to high byte ?
         incf    tadri+2     ;increment internal address high byte

         popregs savedregs
         gjump   done_read
;
;***********************************************************************
;
;   Subroutine F18_BUF_SETUP
;
;   Set up for writing to the write buffer for a panel.  TADR is the
;   start address of the 8 byte region that will be written with this
;   buffer full.  Results are undefined if the low 3 bits of TADR are
;   not zero.
;
         glbsub  f18_buf_setup, regf0 | regf1

         writing             ;indicate that the target is being written to

         core_instr h'8EA6'  ;BSF  EECON1, EEPGD
         core_instr h'9CA6'  ;BCF  EECON1, CFGS

         mcall   adr18f      ;set the write start address

         leaverest

         end
