;   ***************************************************************
;   * Copyright (c) 2004, Embed Inc (http://www.embedinc.com)     *
;   *                                                             *
;   * Permission to copy this file is granted as long as this     *
;   * copyright notice is included in its entirety at the         *
;   * beginning of the file, whether the file is copied in whole  *
;   * or in part and regardless of whether other information is   *
;   * added to the copy.                                          *
;   *                                                             *
;   * The contents of this file may be used in any way,           *
;   * commercial or otherwise.  This file is provided "as is",    *
;   * and Embed Inc makes no claims of suitability for a          *
;   * particular purpose nor assumes any liability resulting from *
;   * its use.                                                    *
;   ***************************************************************
;
;   Host command processor.  The routines in this module interpret and
;   process the command stream coming from the host computer via the
;   RS-232 link.
;
         include "prg.inc"

         extern  uart_get    ;get next serial line input byte into REG0
         extern  uart_put    ;send the byte in REG0 to the host
         extern  op_off      ;turn of power and prog voltage to the target chip
         extern  sert_send   ;send REG8 low bits of REGA to the target
         extern  sert_recv   ;get REG8 bits from target into low bits of REGA
         extern  sert_test_tdrive ;set FLAG_TDRIVE is target is driving PGD line
         extern  vdd_off     ;turn off Vdd power to the target chip
         extern  vdd_low     ;set target chip power voltage to low level
         extern  vdd_norm    ;set target chip power voltage to normal level
         extern  vdd_high    ;set target chip power voltage to high level
         extern  vdd_setvals ;set target chip power voltage levels
         extern  vpp_off     ;turn off target chip programming voltage
         extern  vpp_on      ;turn on target chip programming voltage
         extern  op_reset    ;reset target, leave ready for programming
         extern  op_read     ;read target at TADR into REG1,REG0
         extern  op_write    ;write REG1,REG0 to target at TADR
         extern  op_inc_tadr ;increment desired adr of next operation by 1
         extern  f18_buf_setup ;setup for 18xxx series panel write
         extern  lookinc     ;lookup table entry at REG2:REG1 into REG0

         extern  loop_main   ;return point for event handling routines

         extern  alg_reset   ;1-N algorithm ID for reset
         extern  alg_write   ;1-N algorithm ID for write
         extern  alg_read    ;1-N algorithm ID for read
         extern  tadr        ;desired target address of operation, 24 bits
         extern  tadrr       ;target address after reset
         extern  cntwait     ;wait interval down counter
         extern  tprog       ;wait time after write in timer 2 ticks
         extern_flags        ;declare global flag bits EXTERN
;
;***********************************************************************
;
;   Configuration constants.
;
lbank    equ     1           ;register bank for the local state of this module
;
;   Set MSKSAVE.  This is the bit mask of all the registers that are to
;   be saved accross YIELD.  The remaining registers will be trashed.
;
msksave  equ     regf0 | regf1 | regf2 | regf3 | regf8
;
;   Derived constants.
;
lbankadr equ     bankadr(lbank) ;address within local state register bank
;
;***********************************************************************
;
;   Global state.  All this state is assumed to be in the GBANK register
;   bank by other modules.
;
.bank#v(gbank) udata

;
;***********************************************************************
;
;   Local state.
;
  if lbank != gbank
.bank#v(lbank) udata
    endif

ii       set     1           ;init register number
  while ii < numregs
    if (1 << ii) & msksave   ;save/restore this register accross YIELD call ?
savereg#v(ii) res 1          ;make save area for this register
      endif
ii       set     ii + 1      ;advance to next register number
    endw

restad   res     2           ;thread restart address

.cmd     code
;
;***********************************************************************
;
;   Subroutine CMD_INIT
;
;   Initialize the state managed by this module.
;
         glbsub  cmd_init, noregs
;
;   The INIT and RESET operations are the same thing.  This routine
;   therefore falls thru onto the RESET routine.
;
;***********************************************************************
;
;   Subroutine CMD_RESET
;
;   Reset the command processing state.  This routine is called when host
;   communication times out.  The next input byte will be interpreted as
;   the start of a new communications session.
;
         glbsub  cmd_reset, noregs

         dbankif lbankadr
         movlw   low cmd_start ;init to where to start processing next in byte
         movwf   restad+0
         movlw   high cmd_start
         movwf   restad+1

         dbankif gbankadr
         bcf     flag_waitcmd ;don't run thread until an input byte available

         leaverest
;
;***********************************************************************
;
;   Local subroutine OPC_LOOKUP
;
;   Look up the opcode in REG0 in the opcodes dispatch table TBL_CMD at
;   the end of this module.  The address of the opcode processing routine
;   is returned in REG2:REG1 if an opcode processing routine exists.
;   If not, then REG2:REG1 is returned zero.
;
         locsub  opc_lookup, regf0

         clrf    reg1        ;init return value to indicate invalid opcode
         clrf    reg2

         movf    reg0, w     ;get the opcode
         skip_nz             ;this is not the special opcode 0 ?
         jump    opcl_leave  ;special opcode 0 is always invalid
         sublw   tbl_cmd_n   ;compare to last valid command ID
         skip_wle            ;this opcode has a table entry ?
         jump    opcl_leave  ;no
;
;   There is a dispatch table entry for this opcode.  The opcode is in REG0.
;
         decf    reg0, w     ;make 0-N entry number within table
         addlw   low tbl_cmd ;init table entry address low byte
         movwf   reg1
         movlw   high tbl_cmd ;init table entry address high byte
         movwf   reg2
         skip_ncarr          ;no carry into high byte ?
         incf    reg2        ;propagate the carry

         decf    reg0, w     ;get 0-N table entry number again
         addwf   reg1        ;add it again to entry address low byte
         skip_ncarr          ;no carry to high byte ?
         incf    reg2        ;propagate the carry
;
;   The address of the dispatch table entry for this opcode is in REG2:REG1.
;
         gcallr  lookinc, reg0 ;get opcode routine address low byte into REG0
         gcallr  lookinc, reg2 ;get opcode routine address high byte into REG2
         movf    reg0, w     ;return the routine address low byte in REG1
         movwf   reg1

opcl_leave unbank            ;common exit point
         leaverest
;
;***********************************************************************
;
;   Macro PUTBYTE <bval>
;
;   Send the byte BVAL to the host.  This is a simple convenience wrapper
;   that sets REG0 to the byte value, then calls UART_PUT.  REG0 is left
;   set to the byte value.
;
putbyte  macro   bval
         movlw   (bval)      ;get the byte value into W
         movwf   reg0        ;pass it in REG0
         gcall   uart_put    ;send it to the host
         endm
;
;***********************************************************************
;
;   Subroutine SEND_I16
;
;   Send the 16 bit value in REG1,REG0 to the host.
;
         locsub  send_i16, regf0

         gcall   uart_put    ;send the low byte
         movf    reg1, w     ;get the high byte
         movwf   reg0        ;pass it
         gcall   uart_put    ;send it

         leaverest
;
;***********************************************************************
;
;   Macro YIELD
;
;   Yield the processor to other tasks from.  This macro is only called
;   from the thread.  Only the registers indicated by MSKSAVE (above)
;   are saved accross YIELD calls.
;
;   The restart address immediately after the macro is saved, the MSKSAVE
;   registers are saved, and a hidden return is performed.  The thread
;   is restarted immediately after this macro on the next call to
;   CMD_THREAD.
;
yield    macro
         local   restart
;
;   Set the address at which to restart next time.  This is the address
;   immediately after this macro.
;
         dbankif lbankadr
         movlw   low restart ;save where to restart next time
         movwf   restad+0
         movlw   high restart
         goto    yield2      ;to non-replicated code to do the rest
;
;   Set the assembler state to indicate the register bank settings.
;   The settings are a function of the code in CMD_THREAD that jumps to
;   the restart address.
;
restart                      ;end up here next time CMD_THREAD is called
         dbankis lbankadr    ;direct register set for access to local state
         ibank?              ;indirect bank setting is unknown
         endm
;
;**********
;
;   This section of code is only run implicitly from the YIELD macro.
;   There is only one copy of this code at a fixed location, whereas
;   the YIELD code is replicated for every use of the macro.  This code
;   is jumped to from YIELD to perform as much of the YIELD operation
;   as possible that is not unique to each individual invocation of YIELD.
;   This reduces redundant code which would otherwise be produced for each
;   YIELD invocation.
;
yield2
;
;   The low byte of the restart address has already been saved in RESTAD+0,
;   and the high byte is in W.  Now save the high byte into RESTAD+1.
;   The direct register bank is set for access to the local state.
;
         dbankis lbankadr
         movwf   restad+1
;
;   Save some of the registers locally.  These will be restored when
;   CMD_THREAD is called next before it jumps to the restart address.
;
         dbankif lbankadr
ii       set     1           ;init register number
  while ii < numregs
    if (1 << ii) & msksave   ;save/restore this register accross YIELD call ?
         movf    reg#v(ii), w ;get the value of this register
         movwf   savereg#v(ii) ;save it
      endif
ii       set     ii + 1      ;advance to next register number
    endw

         gjump   loop_main   ;back to the main event loop
;
;***********************************************************************
;
;   Macro GETBYTE
;
;   This is a wrapper around YIELD that clears FLAG_WAITCMD before
;   envoking YIELD.  This causes the thread to be suspended until
;   a host input byte is available.  The GETBYTE macro will appear
;   to the caller as returning with the next host input byte in REG0.
;   Otherwise, the registers indicated by MSKSAVE will be preserved
;   with the remaining ones trashed.
;
getbyte  macro
         dbankif gbankadr
         bcf     flag_waitcmd ;indicate to wait until input byte available
         yield
         endm
;
;***********************************************************************
;
;   Routine CMD_THREAD
;
;   This routine is jumped to from the main event loop to run the host
;   command processing thread for a "short" time.
;
;   This routine is only run when one of two event conditions occur,
;   depending on FLAG_WAITCMD.  When FLAG_WAITCMD is reset, then this
;   routine is run when a character from the host is available.  When
;   FLAG_WAITCMD is set, then it is only run when the wait time has
;   expired.
;
;   The event condition is selected in the thread when it releases
;   its time slice.  The YIELD macro will not alter FLAG_WAITCMD, which
;   is kept set by default.  This means YIELD will "return" when
;   not in a wait interval.  The GETBYTE macro resets FLAG_WAITCMD
;   before envoking YIELD, thereby causing a return when the next
;   host byte is available.  The thread restart code will read the
;   host byte into REG0 when FLAG_WAITCMD is reset, then set the
;   flag so that YIELD will work as expected.
;
         glbent  cmd_thread

         dbankif gbankadr
         btfsc   flag_waitcmd ;thread run because input byte available ?
         goto    th_dwait    ;no, run because wait interval ended

         bsf     flag_waitcmd ;default next YIELD to wait for wait interval over
         gcall   uart_get    ;get the input byte into REG0
th_dwait unbank              ;skip to here if not getting byte from host
;
;   Restore the registers that are preserved accross invocations of the
;   YIELD macro.
;
         dbankif lbankadr
ii       set     1           ;init register number
  while ii < numregs
    if (1 << ii) & msksave   ;save/restore this register accross YIELD call ?
         movf    savereg#v(ii), w ;get the saved copy of the register
         movwf   reg#v(ii)   ;restore the register
      endif
ii       set     ii + 1      ;advance to next register number
    endw
;
;   Jump to the restart address.  The restart address was saved in RESTAD
;   by YIELD before it jumped back to the main event loop.
;
         dbankif lbankadr
         movf    restad+1, w ;get restart address high byte
         movwf   pclath      ;set jump address high byte
         movf    restad+0, w ;get restart address low byte
         movwf   pcl         ;jump to the restart address
;
;   The restart address is jumped to with the direct register bank set for
;   access to the local state.  The indirect register bank setting is unknown.
;
;***********************************************************************
;
;   Main command processing thread routine.
;
;   This routine appears to run as an infinite loop to the code here.
;   It is actually run piecemeal from the main event loop when a suitable
;   event occurs.  The return to the main event loop is done by using
;   either the YIELD or GETBYTE macro.  YIELD returns when no wait interval
;   is active, and GETBYTE returns when the next host input byte is available.
;   GETBYTE returns with the host input byte in REG0.
;
cm_nop   unbank              ;NOP command jumps here
cmd_done unbank              ;done processing the current command
cmd_next unbank              ;get and process the next command
         getbyte
cmd_start                    ;initial restart adr set by CMD_INIT, REG0 has opcode
;
;   The new command opcode is in REG0.
;
         mcall   opc_lookup  ;get the opcode routine address into REG2:REG1
         movf    reg1, w     ;set Z if routine address is zero
         iorwf   reg2, w
         skip_nz             ;a processing routine exists for this opcode ?
         jump    cmd_next    ;no, ignore the opcode
;
;   A command processing routine exists for this command.  The command
;   opcode is in REG0, and the command processing routine address in
;   REG2:REG1.
;
;   Now send the ACK for this command so that the host can start sending
;   the next command.  This allows for overlap between the host sending
;   a command and processing the previous one.  If the processing is
;   slow, as during a program write, then the next command will be
;   immediately available when done processing the previous command.
;
         movlw   rsp_ack     ;get the response code
         movwf   reg0        ;pass it
         gcall   uart_put    ;send it to the host
;
;   Pull the I/O subsystem out of tristate mode if required
;
         dbankif gbankadr
         btfss   flag_iosenab ;"I/Os in tristate mode" flag set?
         goto    no_iosenab   ;no, continue

         bcf     flag_iosenab ;I/Os have been pulled out of Hi-Z mode
         dbankif pgc_tris
         bcf     pgc_tris, pgc_bit  ;PGC is an output
         ; leave the rest alone
no_iosenab
;
;   Re-enable the host timeout if it has been disabled
;
         dbankif gbankadr
		 bcf     flag_htimeoutdis
;
;   Jump to the processing routine for this command.  The command processing
;   routine address is in REG2:REG1.
;
         dbankif lbankadr    ;set bank assumed by command processing routine
         movf    reg2, w     ;set jump address high byte
         movwf   pclath
         movf    reg1, w
         movwf   pcl         ;jump to the command processing routine
;
;***********************************************************************
;
;   Macro COMMAND
;
;   Indicate the start of a command processing routine.  This routine
;   is jumped to from the command dispatch code.  This macro allows for
;   setting particular register bank assumptions, etc, as the command
;   dispatch code is modified.
;
;   All registers are undefined at the start of the command.  The command
;   processing routine should eventually jump to CMD_DONE when done
;   processing this command.
;
command  macro
         dbankis lbankadr    ;direct register bank is set for access to local state
         ibank?              ;indirect bank setting is unknown
         endm
;
;***********************************************************************
;
;   Command OFF
;
;   Power off the target chip.
;
cm_off   command
         yield               ;make sure wait interval, if any, has elapsed

         gcall   op_off      ;power down the target chip
         goto    cmd_done    ;done processing this command
;
;***********************************************************************
;
;   Command CM_PINS
;   Ret: <data>
;
;   Returns status that gives some information about how many pins
;   the target chip has.  The bits in DATA mean:
;
;     bit 0 (LSB)  -  1 if chip has 18 or less pins, 0 if more than 18
;
;   All remaining bits are reserved and set to 0.  Reserved bits may
;   have meaning in future versions.
;
; NOT SUPPORTED: The wisp628 doesn't support pin-checking, so we return
;   an "only have 18 pins" response
cm_pins  command
         clrf    reg0        ;init returned data value
         bsf     reg0, 0     ;the chip has 18 or fewer pins

         gcall   uart_put    ;send the data byte
         goto    cmd_done    ;done processing this command
;
;***********************************************************************
;
;   Command SEND1 <N> <data>
;
;   Send up to 8 data bits to the target chip.  N must be 0-7 and
;   is the number of bits to send - 1.  DATA is the data to send, starting
;   with the LSB.
;
cm_send1 command
         getbyte             ;get data bits - 1 into REG0
         movlw   7           ;get mask for valid bits
         andwf   reg0        ;mask in only the valid bits
         incf    reg0, w     ;get number of bits to transfer
         movwf   reg3        ;save number of bits in REG3

         getbyte             ;get the data bits in REG0
         movf    reg3, w     ;pass number of bits to send in REG8
         movwf   reg8
         yield               ;make sure wait interval, if any, has elapsed
         gcall   sert_send   ;send the bits
         goto    cmd_done    ;done processing this command
;
;***********************************************************************
;
;   Command SEND2 <N> <I16 data>
;
;   Send up to 16 data bits to the target chip.  N must be 0-15 and
;   is the number of bits to send - 1.  DATA is the data to send, starting
;   with the LSB.
;
cm_send2 command
         getbyte             ;get data bits - 1 into REG0
         movlw   15          ;get mask for valid bits
         andwf   reg0        ;mask in only the valid bits
         incf    reg0, w     ;get number of bits to transfer
         movwf   reg3        ;save number of bits in REG3

         getbyte             ;get low data bits in REG0
         movf    reg0, w     ;temp save them in REG2
         movwf   reg2
         getbyte             ;get high data bits in REG0
         movf    reg0, w     ;pass high data bits in REG1
         movwf   reg1
         movf    reg2, w     ;pass low data bits in REG0
         movwf   reg0
         movf    reg3, w     ;pass number of bits to send in REG8
         movwf   reg8
         yield               ;make sure wait interval, if any, has elapsed
         gcall   sert_send   ;send the bits

         goto    cmd_done    ;done processing this command
;
;***********************************************************************
;
;   Command RECV1 <n>
;   Ret: <data>
;
;   Read up to 8 data bits from the target chip.  N must be 0-7 and
;   is the number of bits to read - 1.  The bits will be shifted into
;   the LSB of DATA.  Therefore, the last bit read will be in bit 0
;   (LSB), the second to last in bit 1, etc.  Unused bits of data are
;   set to 0.
;
cm_recv1 command
         getbyte             ;get data bits - 1 into REG0
         movlw   7           ;get mask for valid bits
         andwf   reg0        ;mask in only the valid bits
         incf    reg0, w     ;get number of bits to transfer
         movwf   reg8        ;pass number of bits to read
         yield               ;make sure wait interval, if any, has elapsed
         gcall   sert_recv   ;read the bits into REGA

         gcall   uart_put    ;send the data bits to the host
         goto    cmd_done    ;done processing this command
;
;***********************************************************************
;
;   Command RECV2 <n>
;   Ret: <I16 data>
;
;   Read up to 16 data bits from the target chip.  N must be 0-15 and
;   is the number of bits to read - 1.  The bits will be shifted into
;   the LSB of DATA.  Therefore, the last bit read will be in bit 0
;   (LSB), the second to last in bit 1, etc.  Unused bits of data are
;   set to 0.
;
cm_recv2 command
         getbyte             ;get data bits - 1 into REG0
         movlw   15          ;get mask for valid bits
         andwf   reg0        ;mask in only the valid bits
         incf    reg0, w     ;get number of bits to transfer
         movwf   reg8        ;pass number of bits to read
         yield               ;make sure wait interval, if any, has elapsed
         gcall   sert_recv   ;read the bits into REGA

         gcall   uart_put    ;send the data bits low byte
         movf    reg1, w     ;pass the data bits high byte
         movwf   reg0
         gcall   uart_put    ;send the data bits high byte

         goto    cmd_done    ;done processing this command
;
;***********************************************************************
;
;   Command CLKH
;
;   Set the clock line to the target high.
;
cm_clkh  command
         yield               ;make sure wait interval, if any, has elapsed
         dbankif pgc_reg
         bsf     pgc_pin
         goto    cmd_done    ;done processing this command
;
;***********************************************************************
;
;   Command CLKL
;
;   Set the clock line to the target low.
;
cm_clkl  command
         yield               ;make sure wait interval, if any, has elapsed
         dbankif pgc_reg
         bcf     pgc_pin
         goto    cmd_done    ;done processing this command
;
;***********************************************************************
;
;   Command DATH
;
;   Set the data line to the target high.
;
cm_dath  command
         yield               ;make sure wait interval, if any, has elapsed
         dbankif pgd_tris
         bcf     pgd_tris, pgd_bit
         dbankif pgd_reg
         bsf     pgd_pin     ;set the data line high
         goto    cmd_done    ;done processing this command
;
;***********************************************************************
;
;   Command DATL
;
;   Set the data line to the target low.
;
cm_datl  command
         yield               ;make sure wait interval, if any, has elapsed
         dbankif pgd_tris
         bcf     pgd_tris, pgd_bit
         dbankif pgd_reg
         bcf     pgd_pin     ;set the data line low
         goto    cmd_done    ;done processing this command
;
;***********************************************************************
;
;   Command DATR
;   Ret: <data>
;
;   Read the data line from the target.  The result is returned in the
;   low bit of DATA.  Unused DATA bits are set to 0.
;
cm_datr  command
         yield               ;make sure wait interval, if any, has elapsed
         dbankif pgd_tris		;pgd to input mode
         bsf     pgd_tris, pgd_bit
         clrf    reg0        ;init to data line is low
         dbankif pgd_reg
         btfsc   pgd_pin     ;the data line really is low ?
         bsf     reg0, 0     ;no, set the returned bit to 1
         gcall   uart_put    ;send the byte containing the data bit
         goto    cmd_done    ;done processing this command
;
;***********************************************************************
;
;   Command TDRIVE
;   Ret: <data>
;
;   Test whether the target chip is driving the data line.  DATA is
;   returned 1 if the target is driving the line and 0 if the target
;   is in high impedence state.
;
cm_tdrive command
         yield               ;make sure wait interval, if any, has elapsed
         gcall   sert_test_tdrive ;test the target data line impedence
         clrf    reg0        ;init to target is letting data line float
         dbankif gbankadr
         btfsc   flag_tdrive ;target really is letting data line float ?
         bsf     reg0, 0     ;indicate target is driving the data line
         gcall   uart_put    ;send the byte containing the data bit
         goto    cmd_done    ;done processing this command
;
;***********************************************************************
;
;   Command WAIT <I16 200uS>
;
;   Force a minimum wait time before the next operation is performed
;   on the target chip.  The wait time value is an unsigned integer
;   in units of 200uS.  The maximum wait time is therefore 13.1 seconds.
;
cm_wait  command
         getbyte             ;get low wait value in REG0
         movf    reg0, w     ;save it in REG1
         movwf   reg1
         getbyte             ;get high wait value in REG0
         yield               ;make sure any previous wait has elapsed

         intr_off            ;temp disable interrupts
         dbankif gbankadr
         movf    reg1, w     ;set wait counter low byte
         movwf   cntwait+0
         movf    reg0, w     ;set wait counter high byte
         movwf   cntwait+1
         bcf     flag_done   ;reset flag to indicate wait interval in progress}
         intr_on             ;re-enable interrupts

         goto    cmd_done    ;done processing this command
;
;***********************************************************************
;
;   Command FWINFO
;   Ret: <org> <cvlo> <cvhi> <vers> <i32 info>
;
;   Return version and other information about this firmware.  The
;   returned values are:
;
;     ORG  -  1-254 ID of the organization that is responsible for
;       this firmware.  A value of 1 indicates this is in the sequence
;       of official released firmware.  Values of 0 and 255 are
;       reserved.
;
;     CVLO  -  1-254 lowest official version this firmware
;       is backwards compatible with.  0 indicates not compatible with
;       any offical release.  255 is reserved.
;
;     CVHI  -  1-254 highest official version this firmware
;       is compatible with.  0 indicates not compatible with any
;       offical release.  255 is reserved.
;
;     VERS  -  1-254 sequential version number within the responsible
;       organization.  0 and 255 are reserved.
;
;     INFO  -  Arbitrary 32 bits of information private to the
;       organization responsible for this firmware.  This value is
;       always 0 for an official version.
;
cm_fwinfo command
         putbyte info_org    ;send organization ID
         putbyte info_cvlo   ;send backwards compatible version
         putbyte info_cvhi   ;send official version number
         putbyte info_vers   ;send private version number
         putbyte info_info & 255 ;send private 32 bit firmware information
         putbyte (info_info >> 8) & 255
         putbyte (info_info >> 16) & 255
         putbyte (info_info >> 24) & 255
         goto    cmd_done    ;done processing this command
;
;***********************************************************************
;
;   Command VDDVALS <low> <norm> <high>
;
;   Set the low, normal, and high target chip supply voltage levels.
;   Each value is the supply voltage level where 0 - 250 maps to
;   0 - 6 volts.  In other words, the values are in units of 24mV.
;   Each value must be in the range of 0 - 250.
;
cm_vddvals command
         getbyte             ;get low level into REG0
         movf    reg0, w
         movwf   reg3        ;temp save it in REG3

         getbyte             ;get normal level
         movf    reg0, w
         movwf   reg1        ;save it in REG1

         getbyte             ;get high level
         movf    reg0, w
         movwf   reg2        ;save it in REG2

         movf    reg3, w     ;get low level
         movwf   reg0        ;pass it in reg0
         gcall   vdd_setvals ;set the new low, normal, and high Vdd levels
         goto    cmd_done    ;done processing this command
;
;***********************************************************************
;
;   Command VDDLOW
;
;   Set the target chip supply voltage to the "low" level.
;
cm_vddlow command
         yield               ;make sure wait interval, if any, has elapsed
         gcall   vdd_low
         goto    cmd_done    ;done processing this command
;
;***********************************************************************
;
;   Command VDDNORM
;
;   Set the target chip supply voltage to the "normal" level.
;
cm_vddnorm command
         yield               ;make sure wait interval, if any, has elapsed
         gcall   vdd_norm
         goto    cmd_done    ;done processing this command
;
;***********************************************************************
;
;   Command VDDHIGH
;
;   Set the target chip supply voltage to the "high" level.
;
cm_vddhigh command
         yield               ;make sure wait interval, if any, has elapsed
         gcall   vdd_high
         goto    cmd_done    ;done processing this command
;
;***********************************************************************
;
;   Command VDDOFF
;
;   Turn off the target chip power supply.  This sets Vdd to 0.
;
cm_vddoff command
         yield               ;make sure wait interval, if any, has elapsed
         gcall   vdd_off
         goto    cmd_done    ;done processing this command
;
;***********************************************************************
;
;   Command VPPON
;
;   Turn on the target chip programming voltage.
;
cm_vppon command
         yield               ;make sure wait interval, if any, has elapsed
         gcall   vpp_on
         goto    cmd_done    ;done processing this command
;
;***********************************************************************
;
;   Command VPPOFF
;
;   Turn off the target chip programming voltage.
;
cm_vppoff command
         yield               ;make sure wait interval, if any, has elapsed
         gcall   vpp_off
         goto    cmd_done    ;done processing this command
;
;***********************************************************************
;
;   Command IDRESET <id>
;
;   Set the algorithm ID for the RESET operation.
;
cm_idreset command
         getbyte             ;get the ID in REG0
         dbankif gbankadr
         movf    reg0, w
         movwf   alg_reset
         clrf    tadrr+0     ;init the address after reset to 0
         clrf    tadrr+1
         clrf    tadrr+2
         goto    cmd_done    ;done processing this command
;
;***********************************************************************
;
;   Command IDWRITE <id>
;
;   Set the algorithm ID for the WRITE operation.
;
cm_idwrite command
         getbyte             ;get the ID in REG0
         dbankif gbankadr
         movf    reg0, w
         movwf   alg_write
         goto    cmd_done    ;done processing this command
;
;***********************************************************************
;
;   Command IDREAD <id>
;
;   Set the algorithm ID for the READ operation.
;
cm_idread command
         getbyte             ;get the ID in REG0
         dbankif gbankadr
         movf    reg0, w
         movwf   alg_read
         goto    cmd_done    ;done processing this command
;
;***********************************************************************
;
;   Command RESET
;
;   Reset the target chip and leave it ready for programming and other
;   operations.
;
cm_reset command
         dbankif gbankadr
         bcf     flag_spdata ;reset to access program memory, not data space

         yield               ;make sure wait interval, if any, has elapsed
         gcall   op_reset    ;reset the target chip
         yield               ;wait for reset to complete
         gcall   vdd_norm    ;reset to the normal Vdd level

         goto    cmd_done    ;done processing this command
;
;***********************************************************************
;
;   Command TEST1
;
;   This command is a placeholder for running a debugging test.  It is
;   re-written as needed, and should be removed completely from final
;   firmware.
;
cm_test1 command

         gcall   op_reset    ;do a RESET in an infinite loop
         goto    cm_test1


         goto    cmd_done    ;done processing this command
;
;***********************************************************************
;
;   Command ADR <i24 address>
;
;   Set the address for the next target operation.
;
cm_adr   command
         getbyte             ;address byte 0
         dbankif gbankadr
         movf    reg0, w
         movwf   tadr+0

         getbyte             ;address byte 1
         dbankif gbankadr
         movf    reg0, w
         movwf   tadr+1

         getbyte             ;address byte 2
         dbankif gbankadr
         movf    reg0, w
         movwf   tadr+2

         goto    cmd_done    ;done processing this command
;
;***********************************************************************
;
;   Command READ
;   Ret: <i16 data>
;
;   Read from the target chip at the current address and increment
;   the address.  Two bytes of data are always returned.  If the
;   amount of data at one address is less than 16 bits, then the
;   data is justified in the low bits and the unused high bits
;   are set to 0.
;
cm_read  command
         yield               ;make sure wait interval, if any, has elapsed

         gcall   op_read     ;read from current address into REG1,REG0
         mcall   send_i16    ;send the data to the host
         gcall   op_inc_tadr ;increment the desired address for next operation

         goto    cmd_done    ;done processing this command
;
;***********************************************************************
;
;   Command WRITE <i16 data>
;
;   Write to the target at the current address and then increment
;   the address by 1.  The low bits of the 16 bit data word are used
;   when the amount of data at one address is less than 16 bits.
;
cm_write command
         getbyte             ;get low data byte into REG0
         movf    reg0, w     ;temp save it in REG2
         movwf   reg2
         getbyte             ;get high data byte into REG1
         movf    reg0, w     ;move it to REG1
         movwf   reg1
         movf    reg2, w     ;move the low data byte into REG0
         movwf   reg0

         yield               ;make sure wait interval, if any, has elapsed
         gcall   op_write    ;write REG1,REG0 to the current target address
         gcall   op_inc_tadr ;increment the desired address for next operation

         goto    cmd_done    ;done processing this command
;
;***********************************************************************
;
;   Command TPROG <time>
;
;   Sets the programming write cycle delay time in units of timer 2
;   ticks, which are current set to 200uS.  The maximum wait time is
;   therefore 255 * 200uS = 51mS.  Most chips require a wait less than
;   10mS.
;
cm_tprog command
         getbyte             ;get the wait time in REG0
         dbankif gbankadr
         movf    reg0, w     ;set the new programming cycle wait time
         movwf   tprog
         goto    cmd_done    ;done processing this command
;
;***********************************************************************
;
;   Command SPPROG
;
;   Select the program memory space for subsequent read and write
;   operations.
;
cm_spprog command
         dbankif gbankadr
         bcf     flag_spdata ;indicate not in data memory space
         goto    cmd_done    ;done processing this command
;
;***********************************************************************
;
;   Command SPDATA
;
;   Select the data memory space for subsequent read and write
;   operations.
;
cm_spdata command
         dbankif gbankadr
         bsf     flag_spdata ;indicate in data memory space
         goto    cmd_done    ;done processing this command
;
;***********************************************************************
;
;   Command INCADR
;
;   Increment the desired target address by 1.
;
cm_incadr command
         gcall   op_inc_tadr ;increment the address
         goto    cmd_done    ;done processing this command
;
;***********************************************************************
;
;   Command ADRINV
;
;   Invalidate the local copy of the target address.  The target address
;   will be deliberately set before the next operation that assumes
;   an address.
;
cm_adrinv command
         dbankif gbankadr
         bcf     flag_tadrkn ;indicate target address is unknown
         goto    cmd_done    ;done processing this command
;
;***********************************************************************
;
;   Command PAN18 <dat 0> ... <dat 7>
;
;   Write one buffer of 8 bytes to a program memory panel.  This is a
;   specialized command that must only be used if the target chip supports
;   8 byte panel writes (like the 18Fxx2 and 18Fxx8), and the chip is
;   properly set up.  The write will start at the current target address,
;   which must be 8-byte aligned (low 3 bits zero).  The desired target
;   address will be advanced by 8 by this command.  Successive 8 byte
;   panel writes can therefore be performed with only a stream of PAN18
;   commands.
;
cm_pan18 command
         yield               ;make sure wait interval, if any, has elapsed
         gcall   f18_buf_setup ;set up for a new panel write
;
;   Get the 8 data bytes and write them to the panel buffer.  The last write
;   will be performed with an opcode that also starts the programming process.
;
;   Register useage:
;
;     REG1,REG0  -  Data to send to target chip.
;
;     REG2  -  Temp save for data byte.
;
;     REG3  -  Loop counter.  Number of iterations left - 1.
;
;     REG8  -  Number of bits to send to the target.
;
         movlw   3           ;init loop counter, value = iterations - 1
         movwf   reg3
loop_pan18 unbank            ;back here each pair of bytes to write
         getbyte             ;get the low byte
         movf    reg0, w
         movwf   reg2        ;save it in REG2
         getbyte             ;get the high byte
         movf    reg0, w
         movwf   reg1        ;save it in REG1

         movlw   b'1101'     ;get opcode for write 2 and inc address by 2
         movf    reg3        ;set Z if this is last iteration
         skip_nz             ;this is not the last iteration ?
         movlw   b'1111'     ;last iteration, write 2 bytes and start programming
         movwf   reg0        ;pass the bits to send
         movlw   4           ;pass number of bits to send
         movwf   reg8
         gcall   sert_send   ;send the command opcode

         movf    reg2, w     ;get the low data byte
         movwf   reg0        ;pass it
         movlw   16          ;pass number of bits to send
         movwf   reg8
         gcall   sert_send   ;send the two data bytes

         decf    reg3        ;count one less iteration left to go
         btfss   reg3, 7     ;done with all loop iterations ?
         goto    loop_pan18  ;no, back to do next two bytes
;
;   The programming process has been started and must now be timed.  This is
;   done by sending a NOP core instruction, with the 4th clock pulse held
;   high to time the programming operation.
;
         clrf    reg0        ;set value of bits to send
         movlw   3
         movwf   reg8        ;pass number of bits to send
         gcall   sert_send   ;send the first 3 bits of NOP command

         dbankif pgc_reg
         bsf     pgc_pin     ;set the clock line high
         setwaitms 1         ;start a wait for the programming to finish
         ;
         ;   Advance the desired address for the next operation to
         ;   immediately after this block of 8 bytes.  These cycles are
         ;   "free" because we have to wait for the programming to
         ;   complete anyway.
         ;
         dbankif gbankadr
         movlw   8           ;get amount to add to desired address
         addwf   tadr+0      ;add the value to the low byte
         skip_ncarr          ;no carry to middle byte ?
         incf    tadr+1      ;propagate the carry
         skip_nz             ;no carry into high byte ?
         incf    tadr+2      ;propagate the carry
         bcf     flag_tadrkn ;indicate assumed address in TADRI is incorrect

         yield               ;wait for the programming to finish
         dbankif pgc_reg
         bcf     pgc_pin     ;lower the clock line, end the programming operation

         intr_off            ;temp disable interrupts
         dbankif gbankadr
         incf    cntwait     ;set to wait one more clock tick
         bcf     flag_done   ;indicate a wait interval is in progress
         intr_on             ;re-enable interrupts
         yield               ;wait extra clock tick for high voltage discharge time

         clrf    reg0        ;set all the data bits to 0
         clrf    reg1
         movlw   16          ;pass number of bits to send
         movwf   reg8
         gcall   sert_send   ;send the remaining bits of the NOP command

         goto    cmd_done    ;done processing this command
;
;***********************************************************************
;
;   Command RBYTE8
;   Ret: <dat 0> ... <dat 7>
;
;   Read 8 consecutive words and return their low bytes.  The first byte
;   will be read from the desired target address when the command
;   is run.  The desired target address will be left at the next word
;   after the last one that was read.
;
;   The purpose of this command is to provide a faster means of reading
;   large memory areas when the target word size is 8 bits or less.
;
cm_rbyte8 command
         yield               ;make sure wait interval, if any, has elapsed
;
;   Register useage:
;
;     REG1,REG0  -  Word read from target.
;
;     REG2  -  Number of bytes left to send back
;
         movlw   8           ;init loop counter
         movwf   reg2
rb8_loop unbank              ;back here to read and send each new byte
         gcall   op_read     ;read this word into REG1,REG0
         gcall   uart_put    ;send the low byte to the host
         gcall   op_inc_tadr ;increment the desired target address by 1
         decfsz  reg2        ;count one less byte left to do
         goto    rb8_loop    ;back to read and send next byte

         goto    cmd_done    ;done processing this command
;
;***********************************************************************
;
;   Command WRITING
;
;   Indicate that the target is being written to.  This resets the
;   write timeout counter to its maximum time, which causes FLAG_WRITE
;   to be set, which in turn causes the status LED to indicate the
;   target is being written to.
;
;   This is done automatically when higher level write commands are
;   used.  However, low level commands (like SEND and RECV) can be
;   used to write to the target.  The LED is not set to WRITING status
;   by these commands since they can be used for other purposes.
;   The WRITING command provides a means to set the LED to WRITING
;   status explicitly.
;
cm_writing command
         writing             ;indicate currently writing to the target
         goto    cmd_done    ;done processing this command
;
;***********************************************************************
;
;   Command FWINFO2
;   Ret: <firmware ID>
;
;   Send additional information about this firmware.
;
cm_fwinfo2 command
         putbyte info_fwi2   ;return the firmware type to the host
         goto    cmd_done    ;done processing this command
;
;***********************************************************************
;
;   Command RESADR <i24 address>
;
;   Indicate the address the target chip will be set to after a reset.
;
cm_resadr command
         getbyte             ;get low byte and save it in REG1
         movf    reg0, w
         movwf   reg1
         getbyte             ;get middle byte and save it in REG2
         movf    reg0, w
         movwf   reg2
         getbyte             ;get high byte into REG0

         dbankif gbankadr
         movf    reg1, w
         movwf   tadrr+0     ;set reset address low byte
         movf    reg2, w
         movwf   tadrr+1     ;set reset address middle byte
         movf    reg0, w
         movwf   tadrr+2     ;set reset address high byte

         goto    cmd_done    ;done processing this command
;
;***********************************************************************
;
;   Command CHKCMD <cmd>
;   Ret: <yes/no>
;
;   Indicates whether command CMD is implemented in this firmware.
;
cm_chkcmd command
         getbyte             ;get the command opcode byte into REG0
         mcall   opc_lookup  ;get address of processing routine for the command
         clrf    reg0        ;init to command is not implemented
         movf    reg1, w     ;make OR of both command routine address bytes
         iorwf   reg2, w
         skip_z              ;no processing routine exists for this command ?
         incf    reg0        ;command is implemented, return 1
         gcall   uart_put    ;send the return byte to the host

         goto    cmd_done    ;done processing this command
;
;***********************************************************************
;
;   Command RUN <Vdd>
;
;   Reset the target PIC and then allow it to run.  After the reset,
;   the Vpp (MCLR), PGC, and PGD lines are set to high impedence,
;   and it is up to the target ciruit to drive them appropriately
;   for the application.  If the VDD parameter byte is 0, then the
;   target Vdd line is also set to high impedence.  If the VDD
;   parameter byte is 1-250, the target Vdd line is driven to that
;   value times 24mV.  In other words, VDD of 0-250 maps to 0.0-6.0
;   volts, except that VDD of 0 is a special case where the target
;   Vdd line is not driven at all.
;
;   The host timeout is disabled by this command, and re-enabled
;   when the next command is received.  This means an application
;   can enter RUN mode and exit without the target system being
;   automatically reset 5 seconds later.
;
cm_run command
         getbyte             ;get the command opcode byte into REG0

         dbankif gbankadr
         bsf     flag_iosenab ;set "I/Os in tristate mode" flag

;
; kill the chargepump
;
         wait                ;make sure any existing wait interval has elapsed
         gcall   vpp_off     ;make sure everthing is powered down
         wait

		 ; vpp is now = 0v = reset mode

;
; disable the host timeout
;
         bsf     flag_htimeoutdis

;
; set target i/o pins Hi-Z, reset mode
;

         dbankif rset_reg
         bsf     rset_pin
         dbankif pgc_tris
         bsf     pgc_tris, pgc_bit
         dbankif pgd_tris
         bsf     pgd_tris, pgd_bit
         dbankif lvpen_tris
         bsf     lvpen_tris, lvpen_bit

         waitms  1           ;1ms in reset

         bcf     rset_pin    ;pull the chip out of reset

         goto    cmd_done    ;done processing this command

;
;***********************************************************************
;
;   Commands dispatch table.
;
;   Each entry is the address of the routine to envoke to service that
;   particular command.  Command 0 is handled as a special case.  The
;   first table entry is for command opcode 1.  There can be at most 255
;   table entries.
;
;   The special command table entry of 0 indicates that the command
;   for that table entry is not implemented, even though the table
;   entry exists.
;
;**********
;
;   Macro TLBENT adr
;
;   Define one table entry.  ADR is the address of the routine to
;   process the command this table entry is for.
;
tlbent   macro   adr
         retlw   low adr
         retlw   high adr
         endm
;
;**********
;
;   Start of table.
;
tbl_cmd
         tlbent  cm_nop      ; 1 - no operation, just responds with ACK
         tlbent  cm_off      ; 2 - power down the target chip
         tlbent  cm_pins     ; 3 - detects whether target has 18 pins or less
         tlbent  cm_send1    ; 4 - send up to 8 bits to the target
         tlbent  cm_send2    ; 5 - send up to 16 bits to the target
         tlbent  cm_recv1    ; 6 - read up to 8 bits from the target
         tlbent  cm_recv2    ; 7 - read up to 16 bits from the target
         tlbent  cm_clkh     ; 8 - raise the clock line to the target
         tlbent  cm_clkl     ; 9 - lower the clock line to the target
         tlbent  cm_dath     ;10 - raise the data line to the target
         tlbent  cm_datl     ;11 - lower the data line to the target
         tlbent  cm_datr     ;12 - read the data line from the target
         tlbent  cm_tdrive   ;13 - test whether target is driving data line
         tlbent  cm_wait     ;14 - specify wait before next op in I16 200uS units
         tlbent  cm_fwinfo   ;15 - get firmware version and other info
         tlbent  cm_vddvals  ;16 - set low, normal, and high Vdd levels
         tlbent  cm_vddlow   ;17 - set Vdd to the low level
         tlbent  cm_vddnorm  ;18 - set Vdd to the normal level
         tlbent  cm_vddhigh  ;19 - set Vdd to the high level
         tlbent  cm_vddoff   ;20 - switch of target chip Vdd (set to 0)
         tlbent  cm_vppon    ;21 - turn on the target chip programming voltage
         tlbent  cm_vppoff   ;22 - turn off the target chip programming voltage
         tlbent  cm_idreset  ;23 - select algorithm for reset
         tlbent  cm_reset    ;24 - reset target, leave ready for programming
         tlbent  cm_idwrite  ;25 - select algorithm for write
         tlbent  cm_idread   ;26 - select algorithm for read
         tlbent  cm_test1    ;27 - debugging test 1
         tlbent  cm_adr      ;28 - set address for next target operation
         tlbent  cm_read     ;29 - read target at curr adr, increment adr
         tlbent  cm_write    ;30 - write to target at curr adr, increment adr
         tlbent  cm_tprog    ;31 - set programming write cycle wait time
         tlbent  cm_spprog   ;32 - select program memory space
         tlbent  cm_spdata   ;33 - select data memory space
         tlbent  cm_incadr   ;34 - increment desired target address by one
         tlbent  cm_adrinv   ;35 - invalidate the assumed target address
         tlbent  cm_pan18    ;36 - perform 8 byte panel write to 18xxx PIC
         tlbent  cm_rbyte8   ;37 - specialized command to read 8 bytes
         tlbent  cm_writing  ;38 - indicate the target is being written to
         tlbent  cm_fwinfo2  ;39 - additional firmware info
         tlbent  cm_resadr   ;40 - set target chip address to assume after reset
         tlbent  cm_chkcmd   ;41 - check for existance of a particular command
; more commands (up to version 8) added by philpem
         tlbent  0           ;42 - power supply voltage for pgmr MCU (ver6, optional)
         tlbent  0           ;43 - get the measured target chip Vdd (ver6, optional)
         tlbent  0           ;44 - get the measured target chip Vpp (ver6, optional)
         tlbent  0           ;45 - wait for a wait interval to elapse (ver7, optional) ** TODO: implement
         tlbent  0           ;46 - get number of button presses since powerup (ver8, optional)
         tlbent  0           ;47 - application led status (ver8, optional)
         tlbent  cm_run      ;48 - reset the target and allow it to run (ver8, optional)

tbl_cmd_n equ    ($ - tbl_cmd) >> 1 ;number of entries in TBL_CMD

         end
