;   ***************************************************************
;   * Copyright (C) 2003, Embed Inc (http://www.embedinc.com)     *
;   *                                                             *
;   * Permission to copy this file is granted as long as this     *
;   * copyright notice is included in its entirety at the         *
;   * beginning of the file, whether the file is copied in whole  *
;   * or in part and regardless of whether other information is   *
;   * added to the copy.                                          *
;   *                                                             *
;   * The contents of this file may be used in any way,           *
;   * commercial or otherwise.  This file is provided "as is",    *
;   * and Embed Inc makes no claims of suitability for a          *
;   * particular purpose nor assumes any liability resulting from *
;   * its use.                                                    *
;   ***************************************************************
;
         include "prg.inc"

         extern  done_read   ;jump here when specific read routine done
         extern  done_write  ;jump here when specific write routine done

         extern  tadr        ;requested target chip address of operation
         extern  tadri       ;actual internal target chip address
         extern  tadrr       ;target chip address after reset
         extern  cntwait     ;wait interval down counter
         extern  tprog       ;wait time after write in timer 2 ticks
         extern_flags        ;declare global flag bits EXTERN
;
;***********************************************************************
;
;   Configuration constants.
;
lbank    equ     0           ;register bank for the local state of this module
;
;   Derived constants.
;
lbankadr equ     bankadr(lbank) ;address within local state register bank
;
;***********************************************************************
;
;   Global state.  All this state is assumed to be in the GBANK register
;   bank by other modules.
;
         defram  gbankadr

;
;***********************************************************************
;
;   Local state.
;
         defram  lbankadr

.core12  code
;
;***********************************************************************
;
;   Local subroutine SET_ADR
;
;   Set the internal target chip address to match the desired address.
;   The desired address is in TADR and the current address internal
;   to the target chip is TADRI.  TADRI is only valid (target address
;   is known) if FLAG_TADRKN is set.  TADRR is the address the target
;   chip will be set to on reset, and is the last address of the
;   program memory space.
;
;   This version is for the 12 bit core, and only the two low bytes of
;   TADR and TADRI are used.
;
         locsub  set_adr, regf0 | regf1
;
;   Flag the address as unknown if the target is the configuration word, and
;   the target address has been incremented since reset.  The configuration
;   word can only be accessed before any address increment.  The target
;   chip address is the configuration word address after a reset.
;
         dbankif gbankadr
         movf    tadr+0, w
         xorwf   tadrr+0, w
         movwf   reg0
         movf    tadr+1, w
         xorwf   tadrr+1, w
         iorwf   reg0
         movf    tadr+2, w
         xorwf   tadrr+2, w
         iorwf   reg0
         skip_z              ;target address is the configuration word ?
         jump    done_config ;no
         ;
         ;   The desired address is the configuration word address.
         ;
         btfss   flag_adrinc ;target address incremented since last reset ?
         jump    done_config ;no, target ready for config word address
         bcf     flag_tadrkn ;target incremented, invalidate address assumption
done_config                  ;done checking for config word special case
;
;   Reset the target chip if its address is unknown.
;
         dbankif gbankadr
         btfsc   flag_tadrkn ;target address is unknown ?
         jump    sa_dreset   ;target address is known, skip the reset

         wait
         gcall   op_reset    ;reset the target chip, curr address becomes TADRR
         wait
sa_dreset                    ;skip to here to avoid the reset
;
;   Mask in only the valid bits of the desired address in TADR.  TADRR is the
;   last valid address, and is a power of 2 minus 1 so is also the mask of
;   valid address bits.
;
         dbankif gbankadr
         movf    tadrr+0, w  ;mask byte 0
         andwf   tadr+0
         movf    tadrr+1, w  ;mask byte 1
         andwf   tadr+1
         clrf    tadr+2      ;byte 2 is always 0
;
;   Keep incrementing the target chip address until it reaches the desired
;   address.
;
sa_loop  unbank
         dbankif gbankadr
         movf    tadri+0, w  ;compare the low bytes
         xorwf   tadr+0, w
         skip_z              ;match ?
         jump    sa_inc      ;no, increment the address
         movf    tadri+1, w  ;compare the high bytes
         xorwf   tadr+1, w
         skip_nz             ;different ?
         jump    sa_leave    ;target is at desired address, all done
         ;
         ;   Increment the target address by one.
         ;
sa_inc   unbank
         movlw   b'000110'   ;send INCREMENT ADDRESS command
         movwf   reg0
         gcall   sert_send6

         gcall   op_inc_tadri ;update our assumed target address
         dbankif gbankadr
         movf    tadrr+0, w  ;mask in only valid bits of new assumed address
         andwf   tadri+0
         movf    tadrr+1, w
         andwf   tadri+1
         clrf    tadri+2

         jump    sa_loop     ;back to test address again

sa_leave unbank              ;common exit point
         leaverest
;
;***********************************************************************
;
;   Routine CORE12_READ
;
;   Read the target chip data at address TADR.  The data is returned in the
;   low bits of REG1,REG0 with unused high bits set to 0.  This routine
;   must jump to DONE_READ when done.  REG2-REG3, REG8 may be trashed.
;
         glbent  core12_read

         mcall   set_adr     ;make sure target is at the desired address

         movlw   b'000100'   ;send READ DATA FROM PROGRAM MEMORY command
         movwf   reg0
         gcall   sert_send6
         gcall   sert_recv14ss ;read the result into REG1:REG0

         gjump   done_read
;
;***********************************************************************
;
;   Subroutine CORE12_WRITE
;
;   Write the data in REG1,REG0 to the target chip at address TADR.
;   This routine must jump to DONE_WRITE when done.  REG0-REG3, REG8
;   may be trashed.
;
         glbent  core12_write

         mcall   set_adr     ;make sure target is at the desired address

         movf    reg0, w     ;save low data byte in REG2
         movwf   reg2
         movlw   2           ;send LOAD DATA FOR PROGRAM MEMORY command
         movwf   reg0
         gcall   sert_send6
         movf    reg2, w     ;restore low data byte into REG0
         movwf   reg0
         gcall   sert_send14ss ;send the data from REG1:REG0

         movlw   8           ;send BEGIN PROGRAMMING command
         movwf   reg0
         gcall   sert_send6

         dbankif gbankadr
         movf    tprog, w    ;get timer 2 ticks to wait for program cycle done
         intr_off            ;temp disable interrupts
         movwf   cntwait+0   ;set the wait time
         clrf    cntwait+1
         bcf     flag_done   ;indicate a wait interval is in progress
         intr_on             ;re-enable interrupts

         writing             ;indicate the target chip is being written to

         wait
         movlw   14          ;send END PROGRAMMING command
         movwf   reg0
         gcall   sert_send6

         gjump   done_write

         end
