;   ***************************************************************
;   * Copyright (c) 2002, Embed Inc (http://www.embedinc.com)     *
;   *                                                             *
;   * Permission to copy this file is granted as long as this     *
;   * copyright notice is included in its entirety at the         *
;   * beginning of the file, whether the file is copied in whole  *
;   * or in part and regardless of whether other information is   *
;   * added to the copy.                                          *
;   *                                                             *
;   * The contents of this file may be used in any way,           *
;   * commercial or otherwise.  This file is provided "as is",    *
;   * and Embed Inc makes no claims of suitability for a          *
;   * particular purpose nor assumes any liability resulting from *
;   * its use.                                                    *
;   ***************************************************************
;
;   Interrupt service and related routines.
;
         include "prg.inc"

         extern  uart_intr_recv ;receive interrupt routine in UART module
         extern  uart_intr_xmit ;transmit interrupt routine in UART module

         extern_flags        ;declare global flag bits EXTERN
;
;***********************************************************************
;
;   Configuration constants.
;
save_fsr equ     true        ;indicate whether ISR must save/restore FSR

;
;   Define the timer 2 setup.  This timer is used for two purposes: to
;   generate the PWM frequency, and to generate periodic interrupts that
;   cause the PWM value to be reloaded and provide time base for the rest
;   of the sofware.  The PWM hardware is used to control the target chip
;   Vdd voltage.
;
;   The instruction clock is divided by the timer 2 prescaler and period
;   register to yield the PWM period.  This is then divided by the postscaler
;   to yield the interrupt period.
;
;   NOTE: The allowable ranges for these values MUST be observed.  Out of
;     range values are not checked but can cause system failure, not just
;     an unexpected PWM slice period.
;
t2pre    equ     1           ;timer 2 prescaler divide value (1, 4, or 16)
t2per    equ     250         ;timer 2 period register divide value (1 - 256)
t2post   equ     4           ;timer 2 postscaler divide value (1 - 16)
;
;   The values above result in the following, assuming 20MHz oscillator
;   which results in a 5MHz instruction clock (Finst).
;
;     Finst / (t2pre * t2per) = 5MHz / (1 * 250) = 5MHz / 250
;       = 20.0KHz PWM frequency (Fpwm)
;     Fpwm / t2post = 20KHz / 4 = 5KHz interrupt rate
;       = 200uS interrupt period (every 1000 instructions)
;
; Timer0 reload value
t0load   equ     (256-250)   ;timer0 reload value
                             ;1:1 scaler = 250 instructions = 20kHz
;
;**********
;
;   Derived constants.
;
t2inst   equ     t2pre * t2per * t2post ;timer 2 interrupt period in instructions
inst_ms  equ     1000000 / nsec_inst ;instructions per millisecond

  if t2pms != (inst_ms / t2inst) ;check that global constant T2PMS is correct
         error   "T2PMS in PRG.INS.ASPIC is inconsistant with value in PRG_INTR.ASPIC."
    endif
  if (t2pms * t2inst) != inst_ms
         error   "Not whole number of timer 2 interrupts per millisecond"
    endif

t2_5ms   equ     t2pms * 5   ;timer 2 interrupts per 5mS
;
;***********************************************************************
;
;   Global state.
;
;   The following global state is in the normal register bank for global
;   state.  The bank is GBANK, and GBANKADR is an address guaranteed to
;   be within this bank.
;
.bank#v(gbank) udata

temp1    res     1           ;temp scratch that may be trashed by any routine

cntwait  res     2           ;timer 2 ticks until wait interval over
cn_host  res     1           ;time until disable HOST flag, 10mS units
cn_tron  res     1           ;time until disable TRGON flag, 10mS units
cn_writ  res     1           ;time until disable WRITE flag, 10mS units
cn_hout  res     1           ;time until host comm timeout, 100mS units

         global  temp1
         global  cntwait
         global  cn_host, cn_tron, cn_writ, cn_hout
;
;***********************************************************************
;
;   Local state.  This is always in the same register bank as the global
;   state.
;
cnt1ms   res     1           ;timer 2 ticks until next 1mS tick
cnt10ms  res     1           ;1mS ticks until next 10mS tick
cnt100ms res     1           ;10mS ticks until next 100mS tick
cnt5ms   res     1           ;timer 2 ticks until next 5mS tick
;
;   The following state is private to the interrupt service routine, and
;   must always be in bank 0.
;
  if gbank != 0
.bank0   udata
    endif
status_save res  1           ;saved copy of STATUS, nibbles swapped
  if save_fsr
fsr_save res     1           ;saved copy of FSR (if FSR save enabled)
    endif
  if ncodepages > 1
pclath_save res  1           ;saved copy of PCLATH (if multiple code pages)
    endif

itmp1   res     1            ;temp storage for use within interrupt routine
itmp2   res     1
         global  itmp1, itmp2 ;for use by interrupt code in other modules
;
;   This state is private to the interrupt service routine and must always
;   be accessible regardless of the current direct register bank setting.
;
         udata_shr
w_save   res     1           ;saved W during interrupt, mapped to all banks

.intr    code
;
;***********************************************************************
;
;   Subroutine INTR_INIT
;
;   Initialize the interrupt system and other state managed by this module.
;
         glbsub  intr_init, noregs
;
;   Initialize global state.
;
         dbankif gbankadr
         clrf    cntwait+0   ;init to no wait interval active
         clrf    cntwait+1
         clrf    cn_host     ;init the various timeouts to elapsed
         clrf    cn_tron
         clrf    cn_writ
         clrf    cn_hout
;
;   Initialize local state.
;
         dbankif gbankadr
         movlw   t2_5ms      ;init to maximum time until next 5mS tick
         movwf   cnt5ms
         movlw   t2pms       ;init timer 2 ticks until next 1mS tick
         movwf   cnt1ms
         movlw   10
         movwf   cnt10ms     ;1mS ticks until next 10mS tick
         movwf   cnt100ms    ;10mS ticks until next 100ms tick
;
;   Set up timer 2 to be the PWM time base and to produce the periodic
;   interrupts.
;
         dbankif pr2
         movlw   t2per - 1   ;set period register
         movwf   pr2

         dbankif t2con
t2con_val set    b'00000100' ;set static timer 2 control bits
                 ; X-------  unused
                 ; -0000---  set postscaler to divide value - 1, set below
                 ; -----1--  enable timer 2
                 ; ------00  set prescaler to divide value ID, set below
t2con_val set    t2con_val | ((t2post - 1) << 3) ;merge in postscaler divide select
  if t2pre == 4              ;prescaler divide value is 4 ?
t2con_val set    t2con_val | 1 ;merge in value for prescaler divide by 4
    endif
  if t2pre == 16             ;prescaler divide value is 16 ?
t2con_val set    t2con_val | 2 ;merge in value for prescaler divide by 16
    endif
         movlw   t2con_val   ;set timer 2 control register
         movwf   t2con

         dbankif tmr2
         clrf    tmr2        ;leave max time before first interrupt

         dbankif pie1
         bsf     pie1, tmr2ie ;enable timer 2 interrupt

;
;   Set up timer 0 to produce the periodic
;   interrupts.
;
         dbankif tmr0
         movlw   t0load            ;load tmr0's initial value
         movwf   tmr0

         dbankif option_reg
         bcf     option_reg, t0cs  ;tmr0 clocked by Fosc
         bsf     option_reg, psa   ;prescaler connected to wdt, tmr0ps = 1:1
         bcf     option_reg, ps0   ;1:1 prescaler
         bcf     option_reg, ps1
         bcf     option_reg, ps2

;
;   Enable interrupts.
;
         dbankif pir1        ;clear any peripheral interrupt conditions
         clrf    pir1
         bcf     intcon, t0if ;clear timer0 interrupt flag

         bsf     intcon, peie ;allow peripheral interrupts
		 bsf     intcon, t0ie ;allow timer0 interrupts
         bsf     intcon, gie ;globally enable interrupts

         leaverest
;
;***********************************************************************
;
;   Interrupt service routine.
;
;   The processor effectively executes a call to location 4 on an interrupt,
;   except that global interrupts are also disabled.  These are re-enabled
;   at the end of the ISR by the RETFIE instruction.
;
;   Note that subroutine calls must be minimized or avoided in the ISR.
;   Since an interrupt can come at any time in the main code, any additional
;   call stack locations used here are not available anywhere else.
;
.intr_svc code   4           ;start at interrupt vector location
         movwf   w_save      ;save W
         swapf   status, w   ;make copy of status without effecting status bits
         clrf    status      ;select direct and indirect register banks 0
         dbankis 0
         ibankis 0
         movwf   status_save ;save old STATUS value with nibbles swapped

  if save_fsr                ;FSR needs to be saved ?
         movf    fsr, w      ;save FSR
         movwf   fsr_save
    endif

  if ncodepages > 1          ;multiple code pages may be in use ?
         movf    pclath, w   ;save PCLATH
         movwf   pclath_save
         clrf    pclath
    endif
;
;   W, STATUS, FSR (if SAVE_FSR set), and PCLATH (if multiple code pages)
;   have been saved.  Direct and indirect register banks 0 are selected, and
;   the bank assumptions have been set accordingly.  Program memory page 0
;   is selected.
;

;
;   Check for timer 2 periodic interrupt.
;
         dbankif pir1
         btfsc   pir1, tmr2if
         goto    intr_tmr2
;
;   Check for timer 0 periodic interrupt.
;
         dbankif intcon
         btfsc   intcon, t0if
         goto    intr_tmr0
;
;   Check for UART receive interrupt.
;
         dbankif pir1
         btfss   pir1, rcif
         goto    no_uart_recv
         gjump   uart_intr_recv
no_uart_recv dbankis pir1

;
;   Check for UART transmitter ready interrupt.
;
         dbankif pir1
         btfss   pir1, txif
         goto    no_uart_xmit
         gjump   uart_intr_xmit
no_uart_xmit dbankis pir1

         gjump   0           ;unexpected interrupt, should never happen
;
;********************
;
;   Timer 2 interrupt.  These happen T2PMS times per millisecond.
;
intr_tmr2 dbankis pir1
         ibank?
         bcf     pir1, tmr2if ;clear the interrupt condition

;
;   Check for the wait interval is over.  CNTWAIT is the number of timer
;   2 interrupts before the wait interval is over.  There is no wait interval
;   in progress when CNTWAIT is zero.  Otherwise, FLAG_DONE is set when
;   the wait interval expires.
;
         dbankif gbankadr
         movf    cntwait+0, w ;merge both wait counter bytes into W
         iorwf   cntwait+1, w
         skip_nz             ;a wait interval is in progress ?
         goto    done_wait   ;no, skip this section

         movlw   1           ;get amount to subtract from the counter
         subwf   cntwait+0   ;decrement the low byte
         skip_nborr          ;no borrow from the high byte ?
         subwf   cntwait+1   ;propagate the borrow

         movf    cntwait+0, w ;merge both wait counter bytes into W
         iorwf   cntwait+1, w
         skip_nz             ;the wait interval did not just elapse ?
         bsf     flag_done   ;just elapsed, set flag indicating wait is over
done_wait

;
;   Generate the various clock ticks and perform operations required by
;   them.
;
         dbankif gbankadr
         decfsz  cnt1ms      ;one less timer 2 tick until next 1mS
         goto    done_ticks  ;not time for 1mS tick right now
         movlw   t2pms
         movwf   cnt1ms      ;reset tick counter for next tick
;
;   1mS tick.
;
         dbankif gbankadr
         decfsz  cnt10ms     ;count down to next 10mS tick
         goto    done_ticks  ;not time for 10mS tick right now
         movlw   10
         movwf   cnt10ms     ;reset tick counter for next tick
;
;   10mS tick.
;
         dbankif gbankadr
         movf    cn_host     ;set Z if host comm timer zero
         skip_z              ;already zero, nothing to do ?
         decf    cn_host     ;count one less tick until expired

         dbankif gbankadr
         movf    cn_tron     ;set Z if target on timer is zero
         skip_z              ;already zero, nothing to do ?
         decf    cn_tron     ;count one less tick until expired

         dbankif gbankadr
         movf    cn_writ     ;set Z if write timer zero
         skip_z              ;already zero, nothing to do ?
         decf    cn_writ     ;count one less tick until expired

         dbankif gbankadr
         decfsz  cnt100ms    ;count down to next 100mS tick
         goto    done_ticks  ;not time for 100mS tick right now
         movlw   10
         movwf   cnt100ms    ;reset tick counter for next tick
;
;   100mS tick.
;
  if to_hout                 ;host activity timeout enabled ?
; DISABLED: debug mode
;         dbankif debug_reg
;         btfsc   debug_pin   ;in normal production, not debug, mode ?
;         goto    done_hout   ;host timeout is disabled in debug mode

         dbankif gbankadr
         btfsc   flag_htimeoutdis  ; host timeout disabled?
         goto    done_hout         ; yes, skip over

         btfss   flag_done   ;not in a deliberate wait interval ?
         goto    done_hout   ;timeout doesn't run during deliberate wait

         movf    cn_hout     ;set Z if host timeout already expired
         skip_nz             ;not already expired
         goto    done_hout   ;timeout expired previously

         decf    cn_hout     ;count one less tick until expired
         skip_nz             ;timeout did not just expire ?
         bsf     flag_htimeout ;expired, indicate host communication timed out
done_hout                    ;done handling host activity timeout
    endif

done_ticks dbankis gbankadr  ;all done handling the various clock ticks
;
;   Update the internal status indicator flags.  These are used to update
;   the LED status indicator from the foreground loop.
;
         dbankif gbankadr

         btfss   flag_vppon  ;target chip Vpp is on ?
         goto    target_off  ;no, the target chip is off
target_on                    ;either target chip Vdd or Vpp is on
         bsf     flag_trgon  ;indicate the target chip is "on"
         movlw   to_tron     ;reset the target chip on indicator timer
         movwf   cn_tron
target_off                   ;skip to here if the target chip is off

         bcf     flag_host   ;init to no host comm in progress
         movf    cn_host     ;set Z on timer expired
         skip_z              ;no host comm in progress ?
         bsf     flag_host   ;host comm is in progress

         bcf     flag_trgon  ;update target chip is on flag
         movf    cn_tron
         skip_z
         bsf     flag_trgon

         bcf     flag_write  ;update target chip write in progress flag
         movf    cn_writ
         skip_z
         bsf     flag_write
;
;********************
;
;   Restore state to when the interrupt occurred and return from interrupt.
;
         glbent  intr_ret    ;common code to return from the interrupt

         clrf    status      ;register bank settings are now 0
         dbankis 0
         ibankis 0
  if ncodepages > 1          ;multiple code pages may be in use ?
         movf    pclath_save, w ;restore PCLATH
         movwf   pclath
    endif

  if save_fsr                ;FSR needs to be restored ?
         movf    fsr_save, w ;restore FSR
         movwf   fsr
    endif

         swapf   status_save, w ;get old STATUS with nibble order restored
         movwf   status      ;restore STATUS, register banks now unknown
         swapf   w_save      ;swap nibbles in saved copy of W
         swapf   w_save, w   ;restore original W

         retfie              ;return from interrupt, re-enable global interrupts

;
;********************
;
;   Timer 0 interrupt.  These happen every 200 instructions
;
intr_tmr0 dbankis pir1
         ibank?
         bcf     intcon, t0if ;clear the interrupt condition
         dbankif tmr0
         movlw   t0load
         movwf   tmr0         ; reload tmr0

;
;   Generate the Vpp supply
;
         dbankif gbankadr
         btfss   flag_vppon       ; is the vpp supply turned on?
         goto    done_pump

         dbankif gbankadr
         btfss   flag_pumpstate   ; pump1_pin = pump_last_value
         goto    pump1_low        ; first pump

         dbankif pump1_reg
		 bsf     pump1_pin
		 goto    pump1_done
pump1_low
         dbankif pump1_reg
         bcf     pump1_pin
pump1_done

         dbankif gbankadr
         btfss   flag_pumpstate   ; pump_last_value = ! pump_last_value
         goto    pump_state_low

         bcf     flag_pumpstate
         goto    pump_state_done
pump_state_low
         bsf     flag_pumpstate
pump_state_done

         dbankif gbankadr
         btfss   flag_pumpstate   ; pump2_pin = pump_last_value
         goto    pump2_low

         dbankif pump2_reg
		 bsf     pump2_pin
		 goto    pump2_done
pump2_low
         dbankif pump2_reg
         bcf     pump2_pin
pump2_done
done_pump
         gjump   intr_ret    ;done handling this interrupt

         end
