;   ***************************************************************
;   * Copyright (c) 2002, Embed Inc (http://www.embedinc.com)     *
;   *                                                             *
;   * Permission to copy this file is granted as long as this     *
;   * copyright notice is included in its entirety at the         *
;   * beginning of the file, whether the file is copied in whole  *
;   * or in part and regardless of whether other information is   *
;   * added to the copy.                                          *
;   *                                                             *
;   * The contents of this file may be used in any way,           *
;   * commercial or otherwise.  This file is provided "as is",    *
;   * and Embed Inc makes no claims of suitability for a          *
;   * particular purpose nor assumes any liability resulting from *
;   * its use.                                                    *
;   ***************************************************************
;
;   This module contains the main application routine that is run after
;   the system is initialized.
;
         include "prg.inc"

         extern  led_update  ;update the LED state to the current internal state

         extern  cmd_thread  ;runs host command processing for a time slice
         extern  op_htimeout ;event routine to handle host timeout

         extern_flags        ;declare global flag bits EXTERN
;
;***********************************************************************
;
;   Configuration constants.
;
lbank    equ     0           ;register bank for the local state of this module
;
;   Derived constants.
;
lbankadr equ     bankadr(lbank) ;address with local state register bank
;
;***********************************************************************
;
;   Local state.
;
.bank#v(lbank) udata

.main    code
;
;***********************************************************************
;
;   Routine MAIN
;
;   This entry point is jumped to from the INIT module, once system
;   initialization is complete.
;
         glbent  main
;
;**********
;
;   Initialize the state managed by this module.
;
         dbankif gbankadr
         bsf     flag_done
;
;**********
;
;   Main application loop.  Back here to look for something to do.
;   Whenever something is found that needs handling, a handler routine
;   is jumped to.  All handler routines jump back to LOOP_MAIN when done.
;   This means events are checked in priority order, with high priority
;   events checked earlier.
;
         glbent  loop_main
;
;   Check for host communications timeout.
;
         dbankif gbankadr
         btfss   flag_htimeout
         goto    not_htout
         gjump   op_htimeout ;run host timeout event handler
not_htout dbankis gbankadr
;
;   Run the host command stream processing thread for a short time
;   if an appropriate event condition exists.  When FLAG_WAITCMD
;   is set the event condition is that the wait interval has elapsed
;   (FLAG_DONE set).  When FLAG_WAITCMD is not set, then the event
;   condition is that there is an input character available from the
;   host (FLAG_SIN set).
;
         dbankif gbankadr
         btfss   flag_waitcmd ;event condition is wait interval is over ?
         goto    ev_sin      ;event condition is host input char is available
         ;
         ;   The event condition is wait interval elapsed.
         ;
         btfss   flag_done   ;the wait interval has elapsed ?
         goto    not_cmd     ;no, don't run the thread now
         goto    do_cmd      ;yes, run the thread for a time slice
         ;
         ;   The event condition is host input char available.
         ;
ev_sin   dbankis gbankadr
         btfss   flag_sin    ;a host input character is available ?
         goto    not_cmd     ;no
do_cmd
         gjump   cmd_thread  ;run host command processing thread for a time slice
not_cmd  unbank
;
;   No events required handling.
;
;   Make sure the LED state reflects the current internal state.
;
         gcall   led_update
;
;   Done with this pass thru the event loop.  Go back and do another pass.
;
         goto    loop_main

         end
