;   ***************************************************************
;   * Copyright (c) 2002, Embed Inc (http://www.embedinc.com)     *
;   *                                                             *
;   * Permission to copy this file is granted as long as this     *
;   * copyright notice is included in its entirety at the         *
;   * beginning of the file, whether the file is copied in whole  *
;   * or in part and regardless of whether other information is   *
;   * added to the copy.                                          *
;   *                                                             *
;   * The contents of this file may be used in any way,           *
;   * commercial or otherwise.  This file is provided "as is",    *
;   * and Embed Inc makes no claims of suitability for a          *
;   * particular purpose nor assumes any liability resulting from *
;   * its use.                                                    *
;   ***************************************************************
;
;   Routines that perform higher level operations on the target chip.
;   The code in this module does not control I/O lines directly, but
;   is layered on the routines in the VDD, VPP, SERT, and other
;   modules.
;
         include "prg.inc"

         extern  vpp_off     ;turn off programming voltage, hold MCLR low
         extern  vpp_on      ;apply programming voltage to the target
         extern  vdd_off     ;turn off Vdd power to the target
         extern  vdd_norm    ;apply normal Vdd power to the target
         extern  f16_read    ;target read routine for 16Fxxx
         extern  f18_read    ;target read routine for 18Fxxx
         extern  f16_write   ;target write routine for 16Fxxx
         extern  write_12f6  ;target write routine for 12F6xx
         extern  vdd_get     ;get current Vdd level in REG0
         extern  vdd_set     ;set current Vdd level from REG0

         extern  core12_read ;target read routine for 12 bit core devices
         extern  core12_write ;target write routine for 12 bit core devices
         extern  loop_main   ;return point for event handlers

         extern  cntwait     ;wait interval down counter
         extern_flags        ;declare global flag bits EXTERN
;
;***********************************************************************
;
;   Configuration constants.
;
waitreset equ    5           ;mS to wait after reset
lbank    equ     0           ;register bank for the local state of this module
;
;   Derived constants.
;
lbankadr equ     bankadr(lbank) ;address within local state register bank
;
;***********************************************************************
;
;   Global state.  All this state is assumed to be in the GBANK register
;   bank by other modules.
;
         defram  gbankadr

alg_reset res    1           ;RESET algorithm select
alg_write res    1           ;WRITE algorithm select
alg_read res     1           ;READ algorithm select
tprog    res     1           ;wait time after write, units of timer 2 ticks

tadr     res     3           ;requested target chip address of operation
tadri    res     3           ;actual internal target chip address
tadrr    res     3           ;target address to assume after reset

         global  alg_reset, alg_write, alg_read
         global  tprog
         global  tadr, tadri, tadrr
;
;***********************************************************************
;
;   Local state.
;
         defram  lbankadr

tmp0     res     1           ;local static temporary scratch registers
tmp1     res     1
tmp2     res     1
tmp3     res     1

.op    code
;
;***********************************************************************
;
;   Subroutine OP_INIT
;
;   Initialize the hardware and software state managed by this module.
;
         glbsub  op_init, noregs

         dbankif gbankadr
         clrf    alg_reset   ;init all algorithms to unselected
         clrf    alg_write
         clrf    alg_read

         movlw   10 * t2pms  ;init programming wait time to 10mS
         movwf   tprog

         loadk24 tadr, 0     ;init target chip address of next operation
         loadk24 tadri, 0    ;init current internal target chip address
         loadk24 tadrr, 0    ;init target address after reset

         leaverest
;
;***********************************************************************
;
;   Subroutine OP_INC_TADR
;
;   Increment the desired target address by 1.
;
         glbsub  op_inc_tadr, noregs

         dbankif gbankadr
         incf    tadr+0      ;increment the low byte
         skip_nz             ;no carry into middle byte ?
         incf    tadr+1      ;propagate the carry
         skip_nz             ;no carry into high byte ?
         incf    tadr+2      ;propagate the carry

         leaverest
;
;***********************************************************************
;
;   Subroutine OP_INC_TADRI
;
;   Increment the local copy of the assumed target address by 1.
;
         glbsub  op_inc_tadri, noregs

         dbankif gbankadr
         incf    tadri+0     ;increment the low byte
         skip_nz             ;no carry into middle byte ?
         incf    tadri+1     ;propagate the carry
         skip_nz             ;no carry into high byte ?
         incf    tadri+2     ;propagate the carry

         dbankif gbankadr
         bsf     flag_adrinc ;indicate target address incremented since reset

         leaverest
;
;***********************************************************************
;
;   Macro STARTOP idvar, idmax
;
;   This macro generates the common code at the start of generic operation
;   routines that can be switched to different algorithms.  IDVAR is the
;   variable in the GBANK bank that contains the algorithm ID.
;
;   A table of routine addresses must immediately follow this macro.  This
;   table must have exactly IDMAX entries, with the first entry corresponding
;   to ID 1, the second to ID 2, etc.  If IDVAR is 0 or is greater than
;   IDMAX, then this macro jumps to the instruction immediately following
;   this table.
;
;   This version is for 16 family PIC that can not directly read its own
;   program memory.  Each routine address is stored as two consecutive
;   RETLW instructions in low byte then high byte order.
;
;   All general registers REG0-REGn are preserved.  Local temp scratch
;   registers TMP0 - TMP3 are trashed.
;
         ;
         ;   Set size of each table entry in program memory address units.
         ;
entsize  equ     2           ;two RETLW instructions

startop  macro   idvar, idmax
         local   table
         extern  lookinc     ;lookup table entry at REG2:REG1 into REG0

         dbankif lbankadr
         movf    reg0, w     ;save REGn general registers
         movwf   tmp0
         movf    reg1, w
         movwf   tmp1
         movf    reg2, w
         movwf   tmp2

         dbankif gbankadr
         movf    idvar, w    ;get the algorithm ID
         skip_nz             ;ID is not zero ?
         goto    table + (idmax * entsize) ;ID is zero, abort
         sublw   idmax
         skip_wle            ;ID is within range of the table entries ?
         goto    table + (idmax * entsize) ;no, abort
;
;   The ID value is within range.  Jump to the selected table entry.
;
         dbankif gbankadr
         decf    idvar, w    ;make 0-N entry offset from start of table
         addlw   low table   ;init table entry address low byte
         movwf   reg1
         movlw   high table  ;init table entry address high byte
         movwf   reg2
         skip_ncarr          ;no carry into high byte ?
         incf    reg2        ;propagate the carry

         decf    idvar, w    ;get 0-N table entry number again
         addwf   reg1        ;add it again to entry address low byte
         skip_ncarr          ;no carry to high byte ?
         incf    reg2        ;propagate the carry
;
;   The address of the selected table entry is in REG2:REG1.
;
         gcallr  lookinc, reg0 ;get routine address low byte into TMP3
         dbankif lbankadr
         movf    reg0, w
         movwf   tmp3
         gcallr  lookinc, reg0 ;get routine address high byte into REG0

         dbankif lbankadr
         movf    reg0, w
         movwf   pclath      ;set jump address high byte

         movf    tmp0, w     ;restore REGn general registers
         movwf   reg0
         movf    tmp1, w
         movwf   reg1
         movf    tmp2, w
         movwf   reg2

         movf    tmp3, w     ;jump to the selected operation routine
         movwf   pcl

table                        ;the jump table starts here
         endm
;
;***********************************************************************
;
;   Macro TBLENT adr
;
;   Create one entry in an algorithm routine selection table.  Such a
;   table is intended to start immediately after a STARTOP macro.
;   This macro produces a table entry in the format expected by STARTOP.
;   ADR is the address of the routine to jump to if this entry is
;   selected.
;
tblent   macro   adr
         retlw   low adr
         retlw   high adr
         endm
;
;***********************************************************************
;
;   Subroutine OP_RESET
;
;   Reset the target chip and leave it ready for programming.
;
;   The wait interval must complete after this routine returns before any
;   new target chip operations are attempted.  The current Vdd voltage
;   is preserved.
;
         glbsub  op_reset, regf0 | regf1 | regf2 | regf3 | regf8

         gcall   vdd_get     ;save current Vdd level in REG0

         dbankif gbankadr
         movf    tadrr+0, w  ;init assumed target address after reset
         movwf   tadri+0
         movf    tadrr+1, w
         movwf   tadri+1
         movf    tadrr+2, w
         movwf   tadri+2
         bcf     flag_tadrkn ;init to target address is unknown
         bcf     flag_adrinc ;target address not incremented since reset

         wait                ;make sure any existing wait interval has elapsed
         mcall   op_off      ;make sure everthing is powered down
         wait

		 dbankif lvpen_reg
		 bcf     lvpen_pin
		 dbankif lvpen_tris
		 bcf     lvpen_tris, lvpen_bit   ; LVP pulldown pin low (LVP off for reset)

         startop alg_reset, 3
;
;   Table of routines that implement each of the specific algorithms.
;
         tblent  reset_vdd_vpp ;1 - Vpp on before Vdd, adr reset to TADRR
         tblent  reset_vdd_vpp ;2 - Vdd on before Vpp, adr reset to TADRR
         tblent  reset_vdvp_unk ;3 - Vdd on before Vpp, adr unknown

res_done
		 dbankif lvpen_tris
		 bsf     lvpen_tris, lvpen_bit   ; LVP pulldown pin Hi-Z (LVP on if req'd)

         unbank              ;back here after specific reset algorithm performed
         wait
         gcall   vdd_set     ;restore original Vdd level from REG0
         wait
         setwaitms waitreset

res_leave unbank             ;common exit point for OP_RESET
         leaverest
;
;   RESET_VPP_VDD
;
;   Perform a complete reset and enter programming mode.  The Vpp voltage
;   is raised before the Vdd voltage.  This is required, for example,
;   by the 16F62x.
;
reset_vpp_vdd unbank
         dbankif gbankadr
         bsf     flag_tadrkn ;indicate target address is now known
         gcall   vpp_on      ;turn on the programming voltage
         wait
         gcall   vdd_norm    ;turn on the target chip power
         goto    res_done    ;done with customizable part of reset algorithm
;
;   RESET_VDD_VPP
;
;   Perform a complete reset and enter programming mode.  The Vdd voltage
;   is raised before the Vpp voltage.  This is required, for example,
;   by the 18Fxxx.
;
reset_vdd_vpp unbank         ;target address is reset to 0
         dbankif gbankadr
         bsf     flag_tadrkn ;indicate target address is now known

reset_vdvp_unk unbank        ;target address becomes unknown
         gcall   vdd_norm    ;turn on the target chip power
         wait
         gcall   vpp_on      ;turn on the programming voltage
         goto    res_done    ;done with customizable part of reset algorithm
;
;***********************************************************************
;
;   Subroutine OP_READ
;
;   Read the target chip data at address TADR.  The data is returned in the
;   low bits of REG1,REG0 with unused high bits set to 0.
;
         glbsub  op_read, regf2 | regf3 | regf8

         startop alg_read, 3
;
;   Table of routines that implement each of the specific algorithms.
;
         tblent  f16_read    ;1 - for 16Fxxx
         tblent  f18_read    ;2 - for 18Fxx2 18Fxx8
         tblent  core12_read ;3 - for 12 bit core devices
;
;   Illegal algorithm ID or no algorithm set.
;
         glbent  done_read   ;selected algorithm routines jump here when done
         leaverest
;
;***********************************************************************
;
;   Subroutine OP_WRITE
;
;   Write the data in REG1,REG0 to the target chip at address TADR.
;
;   The wait interval must complete after this routine returns before any
;   new target chip operations are attempted.
;
         glbsub  op_write, regf0 | regf1 | regf2 | regf3 | regf8

         dbankif gbankadr
         bcf     flag_op1    ;reset operation customization flag

         startop alg_write, 3
;
;   Table of routines that implement each of the specific algorithms.
;
         tblent  f16_write   ;1 - for 16Fxxx
         tblent  write_12f6  ;2 - for 12F6xx
         tblent  core12_write ;3 - for 12 bit core devices
;
;   Illegal algorithm ID or no algorithm set.
;
         glbent  done_write  ;selected algorithm routines jump here when done
         leaverest
;
;***********************************************************************
;
;   Subroutine OP_OFF
;
;   Power down and otherwise make it OK for a target chip to be removed or
;   plugged into the socket.
;
;   The wait interval must complete after this routine returns before any
;   new target chip operations are attempted.
;
         glbsub  op_off, noregs

         gcall   vpp_off     ;hold the target in reset
         wait

         dbankif pgc_reg
         bcf     pgc_pin     ;make sure clock line is low
         dbankif pgd_reg
         bcf     pgd_pin     ;make sure data line drive is low
         dbankif pgd_tris
         bcf     pgd_tris, pgd_bit

         gcall   vdd_off     ;power down the target

         leaverest
;
;***********************************************************************
;
;   Event OP_HTIMEOUT
;
;   This routine is jumped to from the main event loop when communication
;   with the host has timed out.  It is assumed that the host has been
;   disconnected or communication otherwise terminated.  This routine
;   makes sure the target is off and the system is in idle mode.
;
;   This routine must jump back to LOOP_MAIN when done.  All registers
;   may be trashed.
;
         glbent  op_htimeout

         dbankif gbankadr
         bcf     flag_htimeout ;clear the event condition
         mcall   op_off      ;turn off all power to target chip
         gjump   loop_main   ;done handling the event

         end
