;   ***************************************************************
;   * Copyright (c) 2002, Embed Inc (http://www.embedinc.com)     *
;   *                                                             *
;   * Permission to copy this file is granted as long as this     *
;   * copyright notice is included in its entirety at the         *
;   * beginning of the file, whether the file is copied in whole  *
;   * or in part and regardless of whether other information is   *
;   * added to the copy.                                          *
;   *                                                             *
;   * The contents of this file may be used in any way,           *
;   * commercial or otherwise.  This file is provided "as is",    *
;   * and Embed Inc makes no claims of suitability for a          *
;   * particular purpose nor assumes any liability resulting from *
;   * its use.                                                    *
;   ***************************************************************
;
;   Low level module for managing the I/O ports.
;
         include "prg.inc"
;
;***********************************************************************
;
;   Configuration constants.
;
lbank    equ     0           ;register bank for the local state of this module
;
;   Derived constants.
;
lbankadr equ     bankadr(lbank) ;address with local state register bank
;
;***********************************************************************
;
;   Local state.
;
.bank#v(lbank) udata

.port    code
;
;***********************************************************************
;
;   Subroutine PORT_INIT
;
;   Initialize the I/O ports as used by the rest of the system.
;
         glbsub  port_init, noregs
;
;   Disable the A/D inputs so that they become digital I/O lines if this
;   processor has an A/D.  If the A/D is used, it will be set up by a
;   module for that purpose later.
;
  ifdef adcon0
         dbankif adcon0
         clrf    adcon0      ;make sure the A/D module is off
    endif

  ifdef adcon1
         dbankif adcon1
         movlw   b'00000110'
                 ; XXXX----  don't matter if disabling the A/D
                 ; ----0110  no A/D inputs, all become digital I/O
         movwf   adcon1
    endif
;
;   Disable the comparator inputs so that they become digital I/O lines if
;   this processor has comparators.  If the comparators are used, they
;   will be set up by a module for that purpose later.
;
  ifdef cmcon
         dbankif cmcon
         movlw   b'00000111'
                 ; XXXXX---  irrelevant when comparators off
                 ; -----111  disable the comparators
         movwf   cmcon
    endif
;
;   Enable or disable the port B weak pullups, depending on the value
;   of PORTBPULLUPS.
;
  if portbpullups
         setreg  b'00000000', option_reg
                 ; 0-------  enable port B passive pullups
                 ; -X------  RB0 interrupt edge select
                 ; --XXXXXX  timer 0 config
    else
         setreg  b'10000000', option_reg
                 ; 1-------  disable port B passive pullups
                 ; -X------  RB0 interrupt edge select
                 ; --XXXXXX  timer 0 config
    endif
;
;   Initialize the I/O ports according to how each of the bits was declared
;   with the /INBIT and /OUTBIT preprocessor directives.
;
  ifdef porta
         dbankif porta
         movlw   val_porta
         movwf   porta
    endif
  ifdef trisa
         dbankif trisa
         movlw   val_trisa
         movwf   trisa
    endif

  ifdef portb
         dbankif portb
         movlw   val_portb
         movwf   portb
    endif
  ifdef trisb
         dbankif trisb
         movlw   val_trisb
         movwf   trisb
    endif

  ifdef portc
         dbankif portc
         movlw   val_portc
         movwf   portc
    endif
  ifdef trisc
         dbankif trisc
         movlw   val_trisc
         movwf   trisc
    endif

  ifdef portd
         dbankif portd
         movlw   val_portd
         movwf   portd
    endif
  ifdef trisd
         dbankif trisd
         movlw   val_trisd
         movwf   trisd
    endif

  ifdef porte
         dbankif porte
         movlw   val_porte
         movwf   porte
    endif
  ifdef trise
         dbankif trise
         movlw   val_trise
         movwf   trise
    endif

  ifdef portf
         dbankif portf
         movlw   val_portf
         movwf   portf
    endif
  ifdef trisf
         dbankif trisf
         movlw   val_trisf
         movwf   trisf
    endif

  ifdef portg
         dbankif portg
         movlw   val_portg
         movwf   portg
    endif
  ifdef trisg
         dbankif trisg
         movlw   val_trisg
         movwf   trisg
    endif

  ifdef porth
         dbankif porth
         movlw   val_porth
         movwf   porth
    endif
  ifdef trish
         dbankif trish
         movlw   val_trish
         movwf   trish
    endif

  ifdef portj
         dbankif portj
         movlw   val_portj
         movwf   portj
    endif
  ifdef trisj
         dbankif trisj
         movlw   val_trisj
         movwf   trisj
    endif
;
;   Initialize other state managed by this module.
;

         leaverest

         end
