;   ***************************************************************
;   * Copyright (c) 2002, Embed Inc (http://www.embedinc.com)     *
;   *                                                             *
;   * Permission to copy this file is granted as long as this     *
;   * copyright notice is included in its entirety at the         *
;   * beginning of the file, whether the file is copied in whole  *
;   * or in part and regardless of whether other information is   *
;   * added to the copy.                                          *
;   *                                                             *
;   * The contents of this file may be used in any way,           *
;   * commercial or otherwise.  This file is provided "as is",    *
;   * and Embed Inc makes no claims of suitability for a          *
;   * particular purpose nor assumes any liability resulting from *
;   * its use.                                                    *
;   ***************************************************************
;
;   Low level routines for performing serial communication with the
;   target chip.
;
;   When the target chip serial interface is not in use, the clock and
;   data lines are left low.  Unless otherwise stated, routines here
;   will expect these conditions on input and leave them that way on
;   output.
;
         include "prg.inc"

         extern_flags        ;declare global flag bits EXTERN
;
;***********************************************************************
;
;   Configuration constants.
;
lbank    equ     0           ;register bank for the local state of this module
;
;   Derived constants.
;
lbankadr equ     bankadr(lbank) ;address within local state register bank
;
;***********************************************************************
;
;   Global state.  All this state is assumed to be in the GBANK register
;   bank by other modules.
;
.bank#v(gbank) udata

;
;***********************************************************************
;
;   Local state.
;
  if lbank != gbank
.bank#v(lbank) udata
    endif

.sert    code
;
;***********************************************************************
;
;   Subroutine SERT_INIT
;
;   Initialize the hardware and software state managed by this module.
;
         glbsub  sert_init, noregs
;
;   Configure the comparators.  An internal comparator is used to decide
;   between the logic high and low values of the PGD pin on the target
;   chip.  A comparator instead of a digital input is used because the
;   supply voltage of the target can vary.  One side of the comparator
;   is connected to the target PGD pin via a resistor, and the other to
;   the output of a voltage divider that produces 1/2 the target Vdd.
;
;         dbankif cmcon
;         movlw   b'00100101' ;configure the comparator module
;                 ; XX------  read-only comparator output bits
;                 ; --1-----  flip C2 output so 1 means target PGD is high
;                 ; ---0----  don't invert comparator 1 output
;                 ; ----X---  input mux unused in this comparator mode
;                 ; -----101  C1 unused, C2 compares RA2 to RA1
;         movwf   cmcon

; More stuff that's useless for the Wisp628...

         leaverest
;
;***********************************************************************
;
;   Subroutine SERT_SEND
;
;   Send raw data bits to the target chip.  The bits are sent from REGA,
;   starting with the LSB.  REG8 indicates the number of bits to send,
;   which must be from 1 to 32.
;
         glbsub  sert_send, regfa | regf8

		 dbankif pgd_tris		; Switch PGD to output mode
		 bcf     pgd_tris, pgd_bit

         dbankif pgd_reg
send_loop                    ;back here to send each new bit
         btfss   rega+0, 0   ;sending a 1 this time ?
         goto    send0bit    ;no, sending a 0 bit
         bsf     pgd_pin     ;set the data line to 1
         goto    done_sendbit
send0bit                     ;the bit to send is a 0
         bcf     pgd_pin     ;set the data line to 0
done_sendbit                 ;done setting the data line to the value of this bit

         dbankif pgc_reg
         bsf     pgc_pin     ;set the clock line high

         rrf     rega+3
         rrf     rega+2
         rrf     rega+1
         rrf     rega+0
         nop                 ;additional settling time
         nop

         bcf     pgc_pin     ;set clock low, clocks data bit into target

         dbankif pgd_reg     ;set bank assumed at start of loop
         decfsz  reg8        ;count one less bit left to do
         goto    send_loop   ;back to send next bit

         dbankif pgd_reg
         bcf     pgd_pin     ;make sure data line is left low

         leaverest
;
;***********************************************************************
;
;   Subroutine SERT_SEND6
;
;   Send the low 6 bits of REG0 to the target, starting with the LSB.
;
         glbsub  sert_send6, regf8

         movlw   6           ;pass number of bits to send
         movwf   reg8
         mcall   sert_send   ;send the bits

         leaverest
;
;***********************************************************************
;
;   Subroutine SERT_SEND14SS
;
;   Send the 14 data bits in the low bits of REG1,REG0 to the target.
;   The data bits will be surrounded by a 0 start and stop bit, for a
;   total of 16 bits sent.
;
         glbsub  sert_send14ss, regf0 | regf1 | regf8

         rlf     reg0        ;move the 14 data bits into position
         rlf     reg1
         bcf     reg0, 0     ;set the start bit to 0
         bcf     reg1, 7     ;set the stop bit to 0

         movlw   16          ;pass the number of bits to send
         movwf   reg8

         mcall   sert_send   ;send the bits

         leaverest
;
;***********************************************************************
;
;   Subroutine SERT_RECV
;
;   Receive raw data bits from the target.  The bits are received in
;   least to most significant order, and are returned in the low bits
;   of REGA with the unused upper bits set to 0.  REG8 indicates the
;   number of bits to receive, which must be from 1 to 32.
;
         glbsub  sert_recv, regf8 | regf9

		 dbankif pgd_tris		; Switch PGD to input mode
		 bsf     pgd_tris, pgd_bit

         clrf    rega+0      ;make sure all unused bits will be returned 0
         clrf    rega+1
         clrf    rega+2
         clrf    rega+3

         movlw   32          ;init number of bits to shift right after receive
         movwf   reg9

         dbankif pgc_reg
recv_loop                    ;back here to receive each new bit
         bsf     pgc_pin     ;set the clock line high

         bcf     status, c   ;shift a 0 bit into MSB of REGA
         rrf     rega+3
         rrf     rega+2
         rrf     rega+1
         rrf     rega+0

         decf    reg9        ;one less bit to shift right after receive

         nop                 ;allow additional time for comparator to settle
         nop
         nop
         nop

         dbankif pgd_reg
         btfsc   pgd_pin       ;the data bit is a 0 as shifted into REGA ?
         bsf     rega+3, 7   ;no, set the new bit in REGA to 1

         dbankif pgc_reg
         bcf     pgc_pin     ;lower the clock line
         nop                 ;additional time before next clock edge

         decfsz  reg8        ;count one less bit left to do
         goto    recv_loop   ;back to receive the next bit
;
;   All the bits have been received into REGA by shifting them in from
;   the MSB.  Now shift REGA right so that the first received bit ends
;   up in the LSB.  REG9 was initialized to 32 and decremented by 1 for
;   each bit shifted into REGA, so REG9 contains the number of bits
;   to shift REGA right to normalize the value in the LSB.
;
         movf    reg9        ;set Z if no more bits left to shift
recv_norm
         skip_nz             ;still need to shift some more ?
         goto    recv_dnorm  ;no, done normalizing the result

         bcf     status, c   ;shift a 0 bit into REGA from the MSB
         rrf     rega+3
         rrf     rega+2
         rrf     rega+1
         rrf     rega+0

         decf    reg9        ;count one less bit left to do
         goto    recv_norm   ;back to check for done

recv_dnorm                   ;done normalizing the value in LSB of REGA

         leaverest
;
;***********************************************************************
;
;   Subroutine SERT_RECV14SS
;
;   Receive 14 data bits from the target, surrounded by a start and stop
;   bit.  The 14 data bits are returned in the low bits of REG1,REG0 with
;   the unused high bits set to 0.
;
         glbsub  sert_recv14ss, regf2 | regf3 | regf8

         movlw   16          ;pass number of data bits to receive
         movwf   reg8
         mcall   sert_recv   ;receive raw 16 bits into REG1,REG0, REG3,REG2 cleared

         rrf     reg1        ;align the 14 data bits into the LSB
         rrf     reg0
         movlw   b'00111111' ;get mask for the data bits in REG1
         andwf   reg1        ;make sure the unused bits in REG1 are 0

         leaverest
;
;***********************************************************************
;
;   Subroutine SERT_TEST_TDRIVE
;
;   Test whether the target chip is driving the serial data line.
;   FLAG_TDRIVE is set if it is and reset if it is not.  The clock line
;   is not altered, so this test has no effect on serial communication
;   protocol state.
;
;   The target is detected to drive the output if the PGD line is ever
;   found opposite of how it is being driven by the PGDOUT output.
;
; TODO: this may cause trouble... ?
         glbsub  sert_test_tdrive, noregs

         dbankif gbankadr
         bsf     flag_tdrive ;init to target is driving the PGD line

		 dbankif pgd_tris		; Switch PGD to input mode
		 bsf     pgd_tris, pgd_bit

         dbankif pgd_reg
         btfss   pgd_pin        ; PGD high?
         goto    tdrive_pgd_low  ; no, it's low
tdrive_pgd_high
         dbankif pgd_reg
		 bcf     pgd_pin        ; pgd is high, drive it low
		 dbankif pgd_tris		; for 1uS
		 bcf     pgd_tris, pgd_bit
		 nop

		 dbankif pgd_tris		; Switch PGD back to input mode
		 bsf     pgd_tris, pgd_bit
         nop
		 nop  ; settling delay
		 nop

		 dbankif pgd_reg
		 btfsc   pgd_pin        ; PGD low = target not driving
		 goto    tdrive_driven
		 goto    done_tdrive    ; PGD high = target driving

tdrive_pgd_low
         dbankif pgd_reg
		 bsf     pgd_pin        ; pgd is low, drive it high
		 dbankif pgd_tris		; for 1uS
		 bcf     pgd_tris, pgd_bit
		 nop

		 dbankif pgd_tris		; Switch PGD back to input mode
		 bsf     pgd_tris, pgd_bit
         nop
		 nop  ; settling delay
		 nop

		 dbankif pgd_reg
		 btfsc   pgd_pin
		 goto    done_tdrive    ; PGD high = target not driving
                                ; PGD low = target driving
tdrive_driven unbank
         dbankif gbankadr
         bsf     flag_tdrive ;indicate target chip is driving PGD line

done_tdrive unbank           ;FLAG_TDRIVE all set, clean up and leave
         dbankif pgd_reg
         bcf     pgd_pin  ;leave PGD drive output low
         leaverest

         end
