;   ***************************************************************
;   * Copyright (c) 2002, Embed Inc (http://www.embedinc.com)     *
;   *                                                             *
;   * Permission to copy this file is granted as long as this     *
;   * copyright notice is included in its entirety at the         *
;   * beginning of the file, whether the file is copied in whole  *
;   * or in part and regardless of whether other information is   *
;   * added to the copy.                                          *
;   *                                                             *
;   * The contents of this file may be used in any way,           *
;   * commercial or otherwise.  This file is provided "as is",    *
;   * and Embed Inc makes no claims of suitability for a          *
;   * particular purpose nor assumes any liability resulting from *
;   * its use.                                                    *
;   ***************************************************************
;
;   Top module for the PRG project.  See the PRG.INS.ASPIC include
;   file for a description of the project.
;
         include "prg.inc"
         extern  regs        ;force general registers to be defined
         extern  stack_init  ;initialize software stack
         extern  port_init   ;initialize I/O ports
         extern  uart_init   ;initialize serial I/O
         extern  cmd_init    ;initialize host command processing
         extern  sert_init   ;initialize target serial interface
         extern  led_init    ;initialize status LED handling
         extern  vdd_init    ;initialize target chip Vdd interface
         extern  vpp_init    ;initialize target chip Vpp interface
         extern  op_init     ;initialize higher level target operations module
         extern  intr_init   ;initialize interrupt system

         extern  main        ;main application routine start point
;
;***********************************************************************
;
;   Set static processor configuration bits for the 16F628.
;
         __config b'11111101000010'
                 ;  1111----------  code protection disabled
                 ;  ----X---------  unused
                 ;  -----1--------  data EEPROM protection diabled
                 ;  ------0-------  low voltage programming disabled
                 ;  -------1------  brown out detection enabled
                 ;  --------0-----  RA5 pin is digital I/O, not MCLR
                 ;  ---------0--10  oscillator mode is high speed crystal
                 ;  ----------0---  power up timer enabled
                 ;  -----------0--  watchdog timer disabled
;
;***********************************************************************
;
;   Global state.
;
.bank#v(gbank) udata
;
;   Declare global flag bytes GFL0 - GFLn.  The assembly constant
;   NFLAGB is set to the number of these flag bytes by the /FLAG
;   preprocessor directives in PRG.INS.ASPIC.
;
         flags_define        ;define the variables for the global flag bits
;
;***********************************************************************
;
;   Executable code.
;
;   Reset vector.
;
.reset   code    0
         clrf    intcon      ;disable all maskable interrupts
         gjump   start       ;jump to relocatable startup code
;
;   Relocatable code.
;
;   This code only initializes the individual modules.  The remaining
;   system initialization is done in the PRG_INIT module, which jumps
;   to MAIN when done.
;
.start   code
start    unbank
;
;   Init the interrupt system to completely off.  INTCON has already been
;   cleared.
;
         ;
         ;   Make sure all interrupts are individually disabled.
         ;
  ifdef pie1
         dbankif pie1
         clrf    pie1
    endif
  ifdef pie2
         dbankif pie2
         clrf    pie2
    endif
         ;
         ;   Clear any existing interrupt conditions.
         ;
  ifdef pir1
         dbankif pir1
         clrf    pir1
    endif
  ifdef pir2
         dbankif pir2
         clrf    pir2
    endif

         flags_clear         ;initialize all global flag bits to 0
;
;   Initialize the separate modules.
;
         gcallnr stack_init  ;init the software stack
         gcallnr port_init   ;init I/O ports
         gcallnr uart_init   ;init serial I/O
         gcallnr cmd_init    ;init host command processing
         gcallnr sert_init   ;init low level target serial communication interface
         gcallnr led_init    ;init status LED driver
         gcallnr vdd_init    ;init target chip Vdd driver
         gcallnr vpp_init    ;init target chip Vpp driver
         gcallnr op_init     ;init module that performs higher level target ops

         gcallnr intr_init   ;init interrupt management and enable interrupts

;
;   All the individual modules have been initialized.  Now start up the
;   overall system.
;
         gjump   main        ;run the main application routine

         end
