;   ***************************************************************
;   * Copyright (c) 2002, Embed Inc (http://www.embedinc.com)     *
;   *                                                             *
;   * Permission to copy this file is granted as long as this     *
;   * copyright notice is included in its entirety at the         *
;   * beginning of the file, whether the file is copied in whole  *
;   * or in part and regardless of whether other information is   *
;   * added to the copy.                                          *
;   *                                                             *
;   * The contents of this file may be used in any way,           *
;   * commercial or otherwise.  This file is provided "as is",    *
;   * and Embed Inc makes no claims of suitability for a          *
;   * particular purpose nor assumes any liability resulting from *
;   * its use.                                                    *
;   ***************************************************************
;
;   The routines in this module provide a procedural interface for
;   controlling Vdd of the target chip.
;
         include "prg.inc"

         extern  cntwait     ;wait interval down counter

         extern_flags        ;declare global flag bits EXTERN
;
;***********************************************************************
;
;   Configuration constants.
;

lbank    equ     0           ;register bank for the local state of this module
;
;   Derived constants.
;
lbankadr equ     bankadr(lbank) ;address within local state register bank
;
;***********************************************************************
;
;   Global state.  All this state is assumed to be in the GBANK register
;   bank by other modules.
;
.bank#v(gbank) udata

;
;***********************************************************************
;
;   Local state.
;
  if lbank != gbank
.bank#v(lbank) udata
    endif

.vdd     code
;
;***********************************************************************
;
;   Subroutine VDD_INIT
;
;   Initialize the hardware and software state managed by this module.
;
         glbsub  vdd_init, noregs

         goto    vdd_defaults ;init the Vdd levels for this chip to defaults
;
;***********************************************************************
;
;   Subroutine VDD_DEFAULTS
;
;   Reset the high, normal, and low target chip Vdd levels to default
;   values.
;
; NOT SUPPORTED BY WISP628 HARDWARE
         glbsub  vdd_defaults, noregs

         dbankif lbankadr

         leaverest
;
;***********************************************************************
;
;   Subroutine VDD_OFF
;
;   Turn off the power to the target chip.  The Vdd PWM value is set
;   to zero and both the 18 pin and 40 pin Vdd drivers are disabled.
;
; NOT SUPPORTED BY WISP628 HARDWARE
         glbsub  vdd_off, noregs

         dbankif gbankadr
;         bsf     flag_vddon  ;indicate Vdd is on

leave_off unbank             ;skip to here if Vdd already set this way
         leaverest
;
;***********************************************************************
;
;   Local subroutine VDD_ENABLE
;
;   Enable one of the Vdd drivers, depending on whether the target chip
;   has more than 18 pins or not.
;
; NOT SUPPORTED BY WISP628 HARDWARE
         locsub  vdd_enable, noregs

vden_leave unbank
         leaverest
;
;***********************************************************************
;
;   Subroutine VDD_LOW
;
;   Set the target Vdd voltage to the "low" level for this chip.
;
; NOT SUPPORTED BY WISP628 HARDWARE
         glbsub  vdd_low, noregs

         dbankif gbankadr
;         bsf     flag_vddon  ;indicate Vdd is on

leave_low unbank             ;skip to here if Vdd already set this way
         leaverest
;
;***********************************************************************
;
;   Subroutine VDD_NORM
;
;   Set the target Vdd voltage to the "norm" level for this chip.
;
; NOT SUPPORTED BY WISP628 HARDWARE
         glbsub  vdd_norm, noregs

         dbankif gbankadr
;         bsf     flag_vddon  ;indicate Vdd is on

leave_norm unbank            ;skip to here if Vdd already set this way
         leaverest
;
;***********************************************************************
;
;   Subroutine VDD_HIGH
;
;   Set the target Vdd voltage to the "high" level for this chip.
;
; NOT SUPPORTED BY WISP628 HARDWARE
         glbsub  vdd_high, noregs

         dbankif gbankadr
;         bsf     flag_vddon  ;indicate Vdd is on

leave_high unbank            ;skip to here if Vdd already set this way
         leaverest
;
;***********************************************************************
;
;   Subroutine VDD_SETVALS
;
;   Set the Vdd levels for the target chip.  The values are passed in
;   the registers:
;
;     REG0  -  Value for LOW Vdd voltage
;     REG1  -  Value for NORMAL Vdd voltage
;     REG2  -  Value for HIGH Vdd voltage
;
;   Each value must be from 0 to 250, which is mapped to the range of
;   0 to 6 volts.  The resulting Vdd voltage from one of these
;   values VAL is:
;
;     Vdd = 6 * VAL / 250 volts
;
;   The new values will take effect on the next call to VDD_LOW, VDD_NORM,
;   or VDD_HIGH.
;
; NOT SUPPORTED BY WISP628 HARDWARE

         glbsub  vdd_setvals, noregs

         leaverest
;
;***********************************************************************
;
;   Subroutine VDD_GET
;
;   Return the current Vdd value in REG0.
;
; NOT SUPPORTED BY WISP628 HARDWARE
         glbsub  vdd_get, noregs

         leaverest
;
;***********************************************************************
;
;   Subroutine VDD_SET
;
;   Set the Vdd output voltage to the level indicated by REG0.  The
;   encoding in REG0 is the same as that used by VDD_SETVALS.  This
;   routine changes the voltage only, not the values for high, normal,
;   and low voltage.  The current setting (off, low, normal, high)
;   is invalidated if the new Vdd value differs from the old.  This
;   means that the new value will be lost on the next call to explicitly
;   set Vdd to off, low, normal, or high.
;
; NOT SUPPORTED BY WISP628 HARDWARE
         glbsub  vdd_set, noregs

set_leave unbank
         leaverest

         end
