;   ***************************************************************
;   * Copyright (c) 2004, Embed Inc (http://www.embedinc.com)     *
;   *                                                             *
;   * Permission to copy this file is granted as long as this     *
;   * copyright notice is included in its entirety at the         *
;   * beginning of the file, whether the file is copied in whole  *
;   * or in part and regardless of whether other information is   *
;   * added to the copy.                                          *
;   *                                                             *
;   * The contents of this file may be used in any way,           *
;   * commercial or otherwise.  This file is provided "as is",    *
;   * and Embed Inc makes no claims of suitability for a          *
;   * particular purpose nor assumes any liability resulting from *
;   * its use.                                                    *
;   ***************************************************************
;
         include "prg.inc"

         extern  cntwait     ;wait interval down counter

         extern_flags        ;declare global flag bits EXTERN
;
;***********************************************************************
;
;   Configuration constants.
;
vppunk   equ     0           ;current Vpp setting unknown or invalid, always = 0
vppoff   equ     1           ;target chip Vpp is OFF
vppon    equ     2           ;target chip Vpp is ON

wvppoff  equ     200         ;mS to wait after Vpp shut off
wvppon   equ     200         ;mS to wait after Vpp enabled

lbank    equ     0           ;register bank for the local state of this module
;
;   Derived constants.
;
lbankadr equ     bankadr(lbank) ;address within local state register bank
;
;***********************************************************************
;
;   Global state.  All this state is assumed to be in the GBANK register
;   bank by other modules.
;
.bank#v(gbank) udata

;
;***********************************************************************
;
;   Local state.
;
  if lbank != gbank
.bank#v(lbank) udata
    endif

vppset   res     1           ;ID for current Vpp setting, use VPPxxx symbols

.vpp     code
;
;***********************************************************************
;
;   Subroutine VPP_INIT
;
;   Initialize the hardware and software state managed by this module.
;
         glbsub  vpp_init, noregs

         dbankif lbankadr
         clrf    vppset      ;init Vpp setting to unknown

         mcall   vpp_off     ; turn Vpp off

; put the target into run mode
         dbankif rset_reg
         bcf     rset_pin    ;pull the chip out of reset

         leaverest
;
;***********************************************************************
;
;   Subroutine VPP_OFF
;
;   Turn off the target chip Vpp programming voltage.  This causes the
;   target chip MCLR to go to ground.
;
         glbsub  vpp_off, noregs

         dbankif lbankadr
         movlw   vppoff      ;get desired Vpp state
         xorwf   vppset, w   ;compare to the current state
         skip_nz             ;need to change the state ?
         goto    leave_vppset ;no, nothing more to do

         movlw   vppoff      ;indicate the new state
         movwf   vppset

         dbankif   pump1_tris
         bsf       pump1_tris, pump1_bit
         dbankif   pump2_tris
         bsf       pump2_tris, pump2_bit   ; turn off the charge pump

		 dbankif   rset_reg		; make sure vpp_out is grounded
		 bsf       rset_pin

         setwaitms wvppoff   ;set minimum wait time before next operation

         dbankif gbankadr
         bcf     flag_vppon  ;indicate Vpp is off

leave_vppset                 ;common code to abort from VPP set routine
         leaverest
;
;***********************************************************************
;
;   Subroutine VPP_ON
;
;   Turn on the target chip Vpp programming voltage.  This causes the target
;   chip MCLR to be driven to 13V.
;
         glbsub  vpp_on, noregs

         dbankif lbankadr
         movlw   vppon       ;get desired Vpp state
         xorwf   vppset, w   ;compare to the current state
         skip_nz             ;need to change the state ?
         goto    leave_vppset ;no, nothing more to do

         movlw   vppon       ;indicate the new state
         movwf   vppset

         dbankif   pump1_tris
         bcf       pump1_tris, pump1_bit
         dbankif   pump2_tris
         bcf       pump2_tris, pump2_bit   ; turn on the charge pump

		 dbankif   rset_reg		; make sure vpp_out is enabled
		 bcf       rset_pin

done_vpp_on unbank           ;done turning on the programming voltage
         setwaitms wvppon    ;set minimum wait before next operation

         dbankif gbankadr
         bsf     flag_vppon  ;indicate Vpp is on

         leaverest

         end
